@@ -909,9 +909,9 @@ gimp_core_config_finalize (GObject *object)
   g_free (core_config->plug_in_rc_path);
   g_free (core_config->import_raw_plug_in);
 
-  g_clear_pointer (&core_config->last_known_release, g_free);
+  g_clear_pointer (&core_config->last_known_release,   g_free);
   g_clear_pointer (&core_config->last_release_comment, g_free);
-  g_clear_pointer (&core_config->config_version, g_free);
+  g_clear_pointer (&core_config->config_version,       g_free);
 
   g_clear_object (&core_config->default_image);
   g_clear_object (&core_config->default_grid);
@@ -932,12 +932,12 @@ gimp_core_config_set_property (GObject      *object,
   switch (property_id)
     {
     case PROP_LANGUAGE:
-      g_free (core_config->language);
-      core_config->language = g_value_dup_string (value);
+      g_set_str (&core_config->language,
+                 g_value_get_string (value));
       break;
     case PROP_PREV_LANGUAGE:
-      g_free (core_config->prev_language);
-      core_config->prev_language = g_value_dup_string (value);
+      g_set_str (&core_config->prev_language,
+                 g_value_get_string (value));
       break;
     case PROP_INTERPOLATION_TYPE:
       core_config->interpolation_type = g_value_get_enum (value);
@@ -946,116 +946,116 @@ gimp_core_config_set_property (GObject      *object,
       core_config->default_threshold = g_value_get_int (value);
       break;
     case PROP_PLUG_IN_PATH:
-      g_free (core_config->plug_in_path);
-      core_config->plug_in_path = g_value_dup_string (value);
+      g_set_str (&core_config->plug_in_path,
+                 g_value_get_string (value));
       break;
     case PROP_MODULE_PATH:
-      g_free (core_config->module_path);
-      core_config->module_path = g_value_dup_string (value);
+      g_set_str (&core_config->module_path,
+                 g_value_get_string (value));
       break;
     case PROP_INTERPRETER_PATH:
-      g_free (core_config->interpreter_path);
-      core_config->interpreter_path = g_value_dup_string (value);
+      g_set_str (&core_config->interpreter_path,
+                 g_value_get_string (value));
       break;
     case PROP_ENVIRON_PATH:
-      g_free (core_config->environ_path);
-      core_config->environ_path = g_value_dup_string (value);
+      g_set_str (&core_config->environ_path,
+                 g_value_get_string (value));
       break;
     case PROP_BRUSH_PATH:
-      g_free (core_config->brush_path);
-      core_config->brush_path = g_value_dup_string (value);
+      g_set_str (&core_config->brush_path,
+                 g_value_get_string (value));
       break;
     case PROP_BRUSH_PATH_WRITABLE:
-      g_free (core_config->brush_path_writable);
-      core_config->brush_path_writable = g_value_dup_string (value);
+      g_set_str (&core_config->brush_path_writable,
+                 g_value_get_string (value));
       break;
     case PROP_DYNAMICS_PATH:
-      g_free (core_config->dynamics_path);
-      core_config->dynamics_path = g_value_dup_string (value);
+      g_set_str (&core_config->dynamics_path,
+                 g_value_get_string (value));
       break;
     case PROP_DYNAMICS_PATH_WRITABLE:
-      g_free (core_config->dynamics_path_writable);
-      core_config->dynamics_path_writable = g_value_dup_string (value);
+      g_set_str (&core_config->dynamics_path_writable,
+                 g_value_get_string (value));
       break;
     case PROP_MYPAINT_BRUSH_PATH:
-      g_free (core_config->mypaint_brush_path);
-      core_config->mypaint_brush_path = g_value_dup_string (value);
+      g_set_str (&core_config->mypaint_brush_path,
+                 g_value_get_string (value));
       break;
     case PROP_MYPAINT_BRUSH_PATH_WRITABLE:
-      g_free (core_config->mypaint_brush_path_writable);
-      core_config->mypaint_brush_path_writable = g_value_dup_string (value);
+      g_set_str (&core_config->mypaint_brush_path_writable,
+                 g_value_get_string (value));
       break;
     case PROP_PATTERN_PATH:
-      g_free (core_config->pattern_path);
-      core_config->pattern_path = g_value_dup_string (value);
+      g_set_str (&core_config->pattern_path,
+                 g_value_get_string (value));
       break;
     case PROP_PATTERN_PATH_WRITABLE:
-      g_free (core_config->pattern_path_writable);
-      core_config->pattern_path_writable = g_value_dup_string (value);
+      g_set_str (&core_config->pattern_path_writable,
+                 g_value_get_string (value));
       break;
     case PROP_PALETTE_PATH:
-      g_free (core_config->palette_path);
-      core_config->palette_path = g_value_dup_string (value);
+      g_set_str (&core_config->palette_path,
+                 g_value_get_string (value));
       break;
     case PROP_PALETTE_PATH_WRITABLE:
-      g_free (core_config->palette_path_writable);
-      core_config->palette_path_writable = g_value_dup_string (value);
+      g_set_str (&core_config->palette_path_writable,
+                 g_value_get_string (value));
       break;
     case PROP_GRADIENT_PATH:
-      g_free (core_config->gradient_path);
-      core_config->gradient_path = g_value_dup_string (value);
+      g_set_str (&core_config->gradient_path,
+                 g_value_get_string (value));
       break;
     case PROP_GRADIENT_PATH_WRITABLE:
-      g_free (core_config->gradient_path_writable);
-      core_config->gradient_path_writable = g_value_dup_string (value);
+      g_set_str (&core_config->gradient_path_writable,
+                 g_value_get_string (value));
       break;
     case PROP_TOOL_PRESET_PATH:
-      g_free (core_config->tool_preset_path);
-      core_config->tool_preset_path = g_value_dup_string (value);
+      g_set_str (&core_config->tool_preset_path,
+                 g_value_get_string (value));
       break;
     case PROP_TOOL_PRESET_PATH_WRITABLE:
-      g_free (core_config->tool_preset_path_writable);
-      core_config->tool_preset_path_writable = g_value_dup_string (value);
+      g_set_str (&core_config->tool_preset_path_writable,
+                 g_value_get_string (value));
       break;
     case PROP_FONT_PATH:
-      g_free (core_config->font_path);
-      core_config->font_path = g_value_dup_string (value);
+      g_set_str (&core_config->font_path,
+                 g_value_get_string (value));
       break;
     case PROP_FONT_PATH_WRITABLE:
-      g_free (core_config->font_path_writable);
-      core_config->font_path_writable = g_value_dup_string (value);
+      g_set_str (&core_config->font_path_writable,
+                 g_value_get_string (value));
       break;
     case PROP_DEFAULT_BRUSH:
-      g_free (core_config->default_brush);
-      core_config->default_brush = g_value_dup_string (value);
+      g_set_str (&core_config->default_brush,
+                 g_value_get_string (value));
       break;
     case PROP_DEFAULT_DYNAMICS:
-      g_free (core_config->default_dynamics);
-      core_config->default_dynamics = g_value_dup_string (value);
+      g_set_str (&core_config->default_dynamics,
+                 g_value_get_string (value));
       break;
     case PROP_DEFAULT_MYPAINT_BRUSH:
-      g_free (core_config->default_mypaint_brush);
-      core_config->default_mypaint_brush = g_value_dup_string (value);
+      g_set_str (&core_config->default_mypaint_brush,
+                 g_value_get_string (value));
       break;
     case PROP_DEFAULT_PATTERN:
-      g_free (core_config->default_pattern);
-      core_config->default_pattern = g_value_dup_string (value);
+      g_set_str (&core_config->default_pattern,
+                 g_value_get_string (value));
       break;
     case PROP_DEFAULT_PALETTE:
-      g_free (core_config->default_palette);
-      core_config->default_palette = g_value_dup_string (value);
+      g_set_str (&core_config->default_palette,
+                 g_value_get_string (value));
       break;
     case PROP_DEFAULT_GRADIENT:
-      g_free (core_config->default_gradient);
-      core_config->default_gradient = g_value_dup_string (value);
+      g_set_str (&core_config->default_gradient,
+                 g_value_get_string (value));
       break;
     case PROP_DEFAULT_TOOL_PRESET:
-      g_free (core_config->default_tool_preset);
-      core_config->default_tool_preset = g_value_dup_string (value);
+      g_set_str (&core_config->default_tool_preset,
+                 g_value_get_string (value));
       break;
     case PROP_DEFAULT_FONT:
-      g_free (core_config->default_font);
-      core_config->default_font = g_value_dup_string (value);
+      g_set_str (&core_config->default_font,
+                 g_value_get_string (value));
       break;
     case PROP_GLOBAL_BRUSH:
       core_config->global_brush = g_value_get_boolean (value);
@@ -1101,8 +1101,8 @@ gimp_core_config_set_property (GObject      *object,
       core_config->undo_preview_size = g_value_get_enum (value);
       break;
     case PROP_PLUGINRC_PATH:
-      g_free (core_config->plug_in_rc_path);
-      core_config->plug_in_rc_path = g_value_dup_string (value);
+      g_set_str (&core_config->plug_in_rc_path,
+                 g_value_get_string (value));
       break;
     case PROP_LAYER_PREVIEWS:
       core_config->layer_previews = g_value_get_boolean (value);
@@ -1134,8 +1134,8 @@ gimp_core_config_set_property (GObject      *object,
       core_config->last_release_timestamp = g_value_get_int64 (value);
       break;
     case PROP_LAST_RELEASE_COMMENT:
-      g_clear_pointer (&core_config->last_release_comment, g_free);
-      core_config->last_release_comment = g_value_dup_string (value);
+      g_set_str (&core_config->last_release_comment,
+                 g_value_get_string (value));
       break;
     case PROP_LAST_REVISION:
       core_config->last_revision = g_value_get_int (value);
@@ -1143,13 +1143,13 @@ gimp_core_config_set_property (GObject      *object,
     case PROP_LAST_KNOWN_RELEASE:
       if (core_config->last_known_release != g_value_get_string (value))
         {
-          g_clear_pointer (&core_config->last_known_release, g_free);
-          core_config->last_known_release = g_value_dup_string (value);
+          g_set_str (&core_config->last_known_release,
+                     g_value_get_string (value));
         }
       break;
     case PROP_CONFIG_VERSION:
-      g_clear_pointer (&core_config->config_version, g_free);
-      core_config->config_version = g_value_dup_string (value);
+      g_set_str (&core_config->config_version,
+                 g_value_get_string (value));
       break;
     case PROP_SAVE_DOCUMENT_HISTORY:
       core_config->save_document_history = g_value_get_boolean (value);
@@ -1168,8 +1168,8 @@ gimp_core_config_set_property (GObject      *object,
       core_config->import_add_alpha = g_value_get_boolean (value);
       break;
     case PROP_IMPORT_RAW_PLUG_IN:
-      g_free (core_config->import_raw_plug_in);
-      core_config->import_raw_plug_in = g_value_dup_string (value);
+      g_set_str (&core_config->import_raw_plug_in,
+                 g_value_get_string (value));
       break;
     case PROP_EXPORT_FILE_TYPE:
       core_config->export_file_type = g_value_get_enum (value);
