@@ -70,6 +70,7 @@ struct _GimpGuiConfig
   gdouble              font_relative_size;
   gboolean             override_icon_size;
   GimpIconSize         custom_icon_size;
+  gboolean             viewables_follow_theme;
   gboolean             use_help;
   gboolean             show_help_button;
   gchar               *help_locales;
@@ -86,6 +87,7 @@ struct _GimpGuiConfig
   gboolean             playground_npd_tool;
   gboolean             playground_seamless_clone_tool;
   gboolean             playground_paint_select_tool;
+  gboolean             playground_use_list_box;
 
   /* saved in sessionrc */
   gboolean             hide_docks;
