@@ -34,6 +34,7 @@
 
 #ifdef G_OS_WIN32
 #include <windows.h>
+#include <process.h>
 #endif
 
 #include <cairo.h>
@@ -146,7 +147,9 @@ gimp_get_default_language (const gchar *category)
 {
   gchar *lang;
   gchar *p;
+#ifndef G_OS_WIN32
   gint   cat = LC_CTYPE;
+#endif
 
   if (! category)
     category = "LC_CTYPE";
@@ -1551,6 +1554,50 @@ gimp_view_size_get_smaller (gint view_size)
   return GIMP_VIEW_SIZE_TINY;
 }
 
+/**
+ * gimp_get_type_children:
+ * @type: the %GType to find children types for.
+ * @types: set %NULL for the initial call (internal variable for
+ *         recursive calls).
+ * @excluded: a list of types to exclude.
+ *
+ * Gather and recursively return all the subtypes of @type (except any
+ * type listed in @excluded).
+ *
+ * Note that @type itself is not included, so if you wished to have it,
+ * you must add it yourself after calling this function.
+ *
+ * Returns: a list of %Gtypes.
+ */
+GList *
+gimp_get_type_children (GType  type,
+                        GList *types,
+                        GList *excluded)
+{
+  GType *dtypes;
+  guint  n_types;
+
+  dtypes = g_type_children (type, &n_types);
+
+  /* TODO: when bumping GLib >= 2.80, use GTYPE_TO_POINTER instead. */
+#define GIMPTYPE_TO_POINTER(t) ((gpointer) (guintptr) (t))
+
+  for (gint i = 0; i < n_types; i++)
+    {
+      if (g_list_find (excluded, GIMPTYPE_TO_POINTER (dtypes[i])))
+        continue;
+
+      types = gimp_get_type_children (dtypes[i], types, excluded);
+      types = g_list_prepend (types, GIMPTYPE_TO_POINTER (dtypes[i]));
+    }
+
+#undef GIMPTYPE_TO_POINTER
+
+  g_free (dtypes);
+
+  return types;
+}
+
 /* Private functions */
 
 
