@@ -15,7 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -59,7 +59,7 @@ gimp_container_filter_foreach_func (GimpObject                 *object,
  * Returns: a weak #GimpContainer filled with matching objects.
  **/
 GimpContainer *
-gimp_container_filter (const GimpContainer  *container,
+gimp_container_filter (GimpContainer        *container,
                        GimpObjectFilterFunc  filter,
                        gpointer              user_data)
 {
@@ -71,8 +71,8 @@ gimp_container_filter (const GimpContainer  *container,
 
   result =
     g_object_new (G_TYPE_FROM_INSTANCE (container),
-                  "children-type", gimp_container_get_children_type (container),
-                  "policy",        GIMP_CONTAINER_POLICY_WEAK,
+                  "child-type", gimp_container_get_child_type (container),
+                  "policy",     GIMP_CONTAINER_POLICY_WEAK,
                   NULL);
 
   context.filter    = filter;
@@ -87,14 +87,13 @@ gimp_container_filter (const GimpContainer  *container,
   if (GIMP_IS_LIST (result))
     gimp_list_reverse (GIMP_LIST (result));
 
-
   return result;
 }
 
 
 static gboolean
-gimp_object_filter_by_name (const GimpObject *object,
-                            const GRegex     *regex)
+gimp_object_filter_by_name (GimpObject   *object,
+                            const GRegex *regex)
 {
   return g_regex_match (regex, gimp_object_get_name (object), 0, NULL);
 }
@@ -111,9 +110,9 @@ gimp_object_filter_by_name (const GimpObject *object,
  * Returns: a weak #GimpContainer filled with matching objects.
  **/
 GimpContainer *
-gimp_container_filter_by_name (const GimpContainer  *container,
-                               const gchar          *regexp,
-                               GError              **error)
+gimp_container_filter_by_name (GimpContainer  *container,
+                               const gchar    *regexp,
+                               GError        **error)
 {
   GimpContainer *result;
   GRegex        *regex;
@@ -138,26 +137,23 @@ gimp_container_filter_by_name (const GimpContainer  *container,
   return result;
 }
 
-
 gchar **
-gimp_container_get_filtered_name_array (const GimpContainer  *container,
-                                        const gchar          *regexp,
-                                        gint                 *length)
+gimp_container_get_filtered_name_array (GimpContainer *container,
+                                        const gchar   *regexp)
 {
   GimpContainer *weak;
   GError        *error = NULL;
 
   g_return_val_if_fail (GIMP_IS_CONTAINER (container), NULL);
-  g_return_val_if_fail (length != NULL, NULL);
 
   if (regexp == NULL || strlen (regexp) == 0)
-    return (gimp_container_get_name_array (container, length));
+    return (gimp_container_get_name_array (container));
 
   weak = gimp_container_filter_by_name (container, regexp, &error);
 
   if (weak)
     {
-      gchar **retval = gimp_container_get_name_array (weak, length);
+      gchar **retval = gimp_container_get_name_array (weak);
 
       g_object_unref (weak);
 
@@ -168,7 +164,41 @@ gimp_container_get_filtered_name_array (const GimpContainer  *container,
       g_warning ("%s", error->message);
       g_error_free (error);
 
-      *length = 0;
       return NULL;
     }
 }
+
+GimpObject **
+gimp_container_get_filtered_array (GimpContainer *container,
+                                   const gchar   *regexp)
+{
+  GimpObject    **retval = NULL;
+  GimpContainer  *weak   = NULL;
+  GError         *error  = NULL;
+
+  g_return_val_if_fail (GIMP_IS_CONTAINER (container), NULL);
+
+  if (regexp != NULL && strlen (regexp) > 0)
+    weak = gimp_container_filter_by_name (container, regexp, &error);
+
+  if (error == NULL)
+    {
+      GList *list;
+      GList *iter;
+      gint   i;
+
+      list   = GIMP_LIST (weak ? weak : container)->queue->head;
+      retval = g_new0 (GimpObject *, g_list_length (list) + 1);
+      for (iter = list, i = 0; iter; iter = iter->next, i++)
+        retval[i] = iter->data;
+    }
+  else
+    {
+      g_warning ("%s", error->message);
+      g_error_free (error);
+    }
+
+  g_clear_object (&weak);
+
+  return retval;
+}
