@@ -27,8 +27,10 @@
 #include "gegl/gimp-gegl-loops.h"
 
 #include "gimpchannel.h"
+#include "core/gimpcontainer.h"
 #include "gimpdrawable.h"
 #include "gimpdrawable-edit.h"
+#include "core/gimpdrawable-filters.h"
 #include "gimpdrawablefilter.h"
 #include "gimpcontext.h"
 #include "gimpfilloptions.h"
@@ -192,7 +194,19 @@ gimp_drawable_edit_fill (GimpDrawable    *drawable,
     {
       gimp_drawable_edit_fill_direct (drawable, options, undo_desc);
 
-      gimp_drawable_update (drawable, x, y, width, height);
+      if (gimp_drawable_has_visible_filters (drawable))
+        {
+          /* For drawables with filters, update the bounding box then
+           * let the drawable update everything, because the filtered
+           * render may be bigger than the filled part.
+           */
+          gimp_drawable_update_bounding_box (drawable);
+          gimp_drawable_update (drawable, 0, 0, -1, -1);
+        }
+      else
+        {
+          gimp_drawable_update (drawable, x, y, width, height);
+        }
     }
   else
     {
@@ -201,6 +215,7 @@ gimp_drawable_edit_fill (GimpDrawable    *drawable,
       gdouble                 opacity;
       GimpLayerMode           mode;
       GimpLayerCompositeMode  composite_mode;
+      GimpContainer          *filter_stack;
 
       opacity        = gimp_context_get_opacity (context);
       mode           = gimp_context_get_paint_mode (context);
@@ -224,7 +239,13 @@ gimp_drawable_edit_fill (GimpDrawable    *drawable,
                                         composite_mode);
 
       gimp_drawable_filter_apply  (filter, NULL);
-      gimp_drawable_filter_commit (filter, NULL, FALSE);
+      /* Move to bottom of filter stack */
+      filter_stack = gimp_drawable_get_filters (drawable);
+      if (filter_stack)
+        gimp_container_reorder (filter_stack, GIMP_OBJECT (filter),
+                                gimp_container_get_n_children (filter_stack) - 1);
+
+      gimp_drawable_filter_commit (filter, FALSE, NULL, FALSE);
 
       g_object_unref (filter);
       g_object_unref (operation);
