@@ -46,11 +46,11 @@
 #include "gimpdrawable-shadow.h"
 #include "gimpdrawable-transform.h"
 #include "gimpdrawablefilter.h"
+#include "gimpdrawablefiltermask.h"
 #include "gimpfilterstack.h"
 #include "gimpimage.h"
 #include "gimpimage-colormap.h"
 #include "gimpimage-undo-push.h"
-#include "gimplayer.h"
 #include "gimpmarshal.h"
 #include "gimppickable.h"
 #include "gimpprogress.h"
@@ -100,6 +100,7 @@ static void       gimp_drawable_get_property       (GObject           *object,
 static gint64     gimp_drawable_get_memsize        (GimpObject        *object,
                                                     gint64            *gui_size);
 
+static void       gimp_drawable_size_changed       (GimpViewable      *viewable);
 static gboolean   gimp_drawable_get_size           (GimpViewable      *viewable,
                                                     gint              *width,
                                                     gint              *height);
@@ -111,6 +112,9 @@ static GeglNode * gimp_drawable_get_node           (GimpFilter        *filter);
 static void       gimp_drawable_removed            (GimpItem          *item);
 static GimpItem * gimp_drawable_duplicate          (GimpItem          *item,
                                                     GType              new_type);
+static void       gimp_drawable_convert            (GimpItem          *item,
+                                                    GimpImage         *dest_image,
+                                                    GType              old_type);
 static void       gimp_drawable_scale              (GimpItem          *item,
                                                     gint               new_width,
                                                     gint               new_height,
@@ -291,6 +295,7 @@ gimp_drawable_class_init (GimpDrawableClass *klass)
 
   gimp_object_class->get_memsize  = gimp_drawable_get_memsize;
 
+  viewable_class->size_changed    = gimp_drawable_size_changed;
   viewable_class->get_size        = gimp_drawable_get_size;
   viewable_class->get_new_preview = gimp_drawable_get_new_preview;
   viewable_class->get_new_pixbuf  = gimp_drawable_get_new_pixbuf;
@@ -301,6 +306,7 @@ gimp_drawable_class_init (GimpDrawableClass *klass)
 
   item_class->removed             = gimp_drawable_removed;
   item_class->duplicate           = gimp_drawable_duplicate;
+  item_class->convert             = gimp_drawable_convert;
   item_class->scale               = gimp_drawable_scale;
   item_class->resize              = gimp_drawable_resize;
   item_class->flip                = gimp_drawable_flip;
@@ -335,7 +341,7 @@ gimp_drawable_init (GimpDrawable *drawable)
 {
   drawable->private = gimp_drawable_get_instance_private (drawable);
 
-  drawable->private->filter_stack = gimp_filter_stack_new (GIMP_TYPE_FILTER);
+  _gimp_drawable_filters_init (drawable);
 }
 
 /* sorry for the evil casts */
@@ -386,7 +392,8 @@ gimp_drawable_finalize (GObject *object)
 
   g_clear_object (&drawable->private->source_node);
   g_clear_object (&drawable->private->buffer_source_node);
-  g_clear_object (&drawable->private->filter_stack);
+
+  _gimp_drawable_filters_finalize (drawable);
 
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
@@ -440,6 +447,36 @@ gimp_drawable_get_memsize (GimpObject *object,
                                                                   gui_size);
 }
 
+static void
+gimp_drawable_size_changed (GimpViewable *viewable)
+{
+  GList *list;
+  gint   width;
+  gint   height;
+
+  if (GIMP_VIEWABLE_CLASS (parent_class)->size_changed)
+    GIMP_VIEWABLE_CLASS (parent_class)->size_changed (viewable);
+
+  width   = gimp_item_get_width (GIMP_ITEM (viewable));
+  height  = gimp_item_get_height (GIMP_ITEM (viewable));
+
+  for (list = GIMP_LIST (GIMP_DRAWABLE (viewable)->private->filter_stack)->queue->tail;
+       list;
+       list = g_list_previous (list))
+    {
+      if (GIMP_IS_DRAWABLE_FILTER (list->data))
+        {
+          GimpDrawableFilter *filter = list->data;
+          GimpChannel        *mask   = GIMP_CHANNEL (gimp_drawable_filter_get_mask (filter));
+          GeglRectangle       rect   = { 0, 0, width, height };
+
+          /* Don't resize partial layer effects */
+          if (gimp_channel_is_empty (mask))
+            gimp_drawable_filter_refresh_crop (filter, &rect);
+        }
+    }
+}
+
 static gboolean
 gimp_drawable_get_size (GimpViewable *viewable,
                         gint         *width,
@@ -529,16 +566,76 @@ gimp_drawable_duplicate (GimpItem *item,
       GimpDrawable  *drawable     = GIMP_DRAWABLE (item);
       GimpDrawable  *new_drawable = GIMP_DRAWABLE (new_item);
       GeglBuffer    *new_buffer;
+      GimpContainer *filters;
+      GList         *list;
 
       new_buffer = gimp_gegl_buffer_dup (gimp_drawable_get_buffer (drawable));
 
       gimp_drawable_set_buffer (new_drawable, FALSE, NULL, new_buffer);
       g_object_unref (new_buffer);
+
+      filters = gimp_drawable_get_filters (drawable);
+
+      for (list = GIMP_LIST (filters)->queue->tail;
+           list;
+           list = g_list_previous (list))
+        {
+          GimpDrawableFilter *filter = list->data;
+
+          if (GIMP_IS_DRAWABLE_FILTER (filter))
+            {
+              GimpDrawableFilter *new_filter;
+
+              new_filter = gimp_drawable_filter_duplicate (new_drawable,
+                                                           filter);
+              if (new_filter)
+                {
+                  gimp_drawable_filter_apply (new_filter, NULL);
+                  gimp_drawable_filter_commit (new_filter, TRUE, NULL, FALSE);
+
+                  gimp_drawable_filter_layer_mask_freeze (new_filter);
+                  g_object_unref (new_filter);
+                }
+            }
+        }
     }
 
   return new_item;
 }
 
+static void
+gimp_drawable_convert (GimpItem  *item,
+                       GimpImage *dest_image,
+                       GType      old_type)
+{
+  GimpContainer *filters;
+
+  filters = gimp_drawable_get_filters (GIMP_DRAWABLE (item));
+  if (gimp_container_get_n_children (filters) > 0)
+    {
+      GList *filter_list;
+
+      for (filter_list = GIMP_LIST (filters)->queue->tail;
+           filter_list;
+           filter_list = g_list_previous (filter_list))
+        {
+          if (GIMP_IS_DRAWABLE_FILTER (filter_list->data))
+            {
+              GimpDrawableFilter     *filter = filter_list->data;
+              GimpDrawableFilterMask *mask;
+
+              mask = gimp_drawable_filter_get_mask (filter);
+
+              if (mask)
+                gimp_drawable_convert (GIMP_ITEM (mask), dest_image,
+                                       GIMP_TYPE_DRAWABLE_FILTER_MASK);
+            }
+        }
+    }
+
+  GIMP_ITEM_CLASS (parent_class)->convert (item, dest_image, old_type);
+}
+
 static void
 gimp_drawable_scale (GimpItem              *item,
                      gint                   new_width,
@@ -570,30 +667,6 @@ gimp_drawable_scale (GimpItem              *item,
                                                  0,            0),
                                  TRUE);
   g_object_unref (new_buffer);
-
-  if (GIMP_IS_LAYER (drawable))
-    {
-      GList *list;
-
-      for (list = GIMP_LIST (drawable->private->filter_stack)->queue->tail;
-           list; list = g_list_previous (list))
-        {
-          if (GIMP_IS_DRAWABLE_FILTER (list->data))
-            {
-              GimpDrawableFilter *filter = list->data;
-              GimpChannel        *mask   = GIMP_CHANNEL (gimp_drawable_filter_get_mask (filter));
-              GeglRectangle      *rect   = GEGL_RECTANGLE (0, 0,
-                                                           new_width,
-                                                           new_height);
-
-              /* Don't resize partial layer effects */
-              if (gimp_channel_is_empty (mask))
-                gimp_drawable_filter_refresh_crop (filter, rect);
-            }
-        }
-      if (list)
-        g_list_free (list);
-    }
 }
 
 static void
@@ -679,28 +752,6 @@ gimp_drawable_resize (GimpItem     *item,
                                                  0,            0),
                                  TRUE);
   g_object_unref (new_buffer);
-
-  if (GIMP_IS_LAYER (drawable))
-    {
-      GList *list;
-
-      for (list = GIMP_LIST (drawable->private->filter_stack)->queue->tail;
-           list; list = g_list_previous (list))
-        {
-          if (GIMP_IS_DRAWABLE_FILTER (list->data))
-            {
-              GimpDrawableFilter *filter = list->data;
-              GimpChannel        *mask   = GIMP_CHANNEL (gimp_drawable_filter_get_mask (filter));
-              GeglRectangle       rect   = {0, 0, new_width, new_height};
-
-              /* Don't resize partial layer effects */
-              if (gimp_channel_is_empty (mask))
-                gimp_drawable_filter_refresh_crop (filter, &rect);
-            }
-        }
-      if (list)
-        g_list_free (list);
-    }
 }
 
 static void
@@ -1176,24 +1227,23 @@ gimp_drawable_update (GimpDrawable *drawable,
 {
   g_return_if_fail (GIMP_IS_DRAWABLE (drawable));
 
-  if (width < 0)
+  if (width < 0 || height < 0)
     {
       GeglRectangle bounding_box;
 
       bounding_box = gimp_drawable_get_bounding_box (drawable);
 
-      x     = bounding_box.x;
-      width = bounding_box.width;
-    }
+      if (width < 0)
+        {
+          x     = bounding_box.x;
+          width = bounding_box.width;
+        }
 
-  if (height < 0)
-    {
-      GeglRectangle bounding_box;
-
-      bounding_box = gimp_drawable_get_bounding_box (drawable);
-
-      y      = bounding_box.y;
-      height = bounding_box.height;
+      if (height < 0)
+        {
+          y      = bounding_box.y;
+          height = bounding_box.height;
+        }
     }
 
   if (drawable->private->paint_count == 0)
@@ -1261,6 +1311,14 @@ gimp_drawable_update_all (GimpDrawable *drawable)
   GIMP_DRAWABLE_GET_CLASS (drawable)->update_all (drawable);
 }
 
+void
+gimp_drawable_filters_changed (GimpDrawable *drawable)
+{
+  g_return_if_fail (GIMP_IS_DRAWABLE (drawable));
+
+  g_signal_emit (drawable, gimp_drawable_signals[FILTERS_CHANGED], 0);
+}
+
 void
 gimp_drawable_invalidate_boundary (GimpDrawable *drawable)
 {
@@ -2086,10 +2144,7 @@ gimp_drawable_end_paint (GimpDrawable *drawable)
   if (gimp_drawable_has_visible_filters (drawable) &&
       drawable->private->paint_count == 0)
     {
-      gimp_item_set_visible (GIMP_ITEM (drawable), FALSE, FALSE);
-      gimp_image_flush (gimp_item_get_image (GIMP_ITEM (drawable)));
-      gimp_item_set_visible (GIMP_ITEM (drawable),TRUE, FALSE);
-      gimp_image_flush (gimp_item_get_image (GIMP_ITEM (drawable)));
+      gimp_drawable_update (drawable, 0, 0, -1, -1);
     }
 
   return result;
@@ -2160,9 +2215,3 @@ gimp_drawable_is_painting (GimpDrawable *drawable)
 
   return drawable->private->paint_count > 0;
 }
-
-void
-gimp_drawable_filters_changed (GimpDrawable *drawable)
-{
-  g_signal_emit (drawable, gimp_drawable_signals[FILTERS_CHANGED], 0);
-}
