@@ -31,7 +31,8 @@ GimpImage    * gimp_image_new_from_drawable     (Gimp            *gimp,
                                                  GimpDrawable    *drawable);
 GimpImage    * gimp_image_new_from_drawables    (Gimp            *gimp,
                                                  GList           *drawables,
-                                                 gboolean         copy_selection);
+                                                 gboolean         copy_selection,
+                                                 gboolean         tag_copies);
 GimpImage    * gimp_image_new_from_component    (Gimp            *gimp,
                                                  GimpImage       *image,
                                                  GimpChannelType  component);
