@@ -36,7 +36,7 @@
 #include "text/gimptext.h"
 #include "text/gimptextlayer.h"
 
-#include "widgets/gimpcontainertreeview.h"
+#include "widgets/gimpcontainerlistview.h"
 #include "widgets/gimplayermodebox.h"
 #include "widgets/gimpviewabledialog.h"
 
@@ -83,6 +83,7 @@ static void   layer_options_dialog_callback       (GtkWidget          *dialog,
                                                    GimpColorTag        item_color_tag,
                                                    gboolean            item_lock_content,
                                                    gboolean            item_lock_position,
+                                                   gboolean            item_lock_visibility,
                                                    gpointer            user_data);
 static void
      layer_options_dialog_update_mode_sensitivity (LayerOptionsDialog *private);
@@ -116,6 +117,7 @@ layer_options_dialog_new (GimpImage                *image,
                           GimpColorTag              layer_color_tag,
                           gboolean                  layer_lock_content,
                           gboolean                  layer_lock_position,
+                          gboolean                  layer_lock_visibility,
                           gboolean                  layer_lock_alpha,
                           GimpLayerOptionsCallback  callback,
                           gpointer                  user_data)
@@ -161,14 +163,16 @@ layer_options_dialog_new (GimpImage                *image,
                                     parent, title, role,
                                     icon_name, desc, help_id,
                                     _("Layer _name:"),
-                                    GIMP_ICON_TOOL_PAINTBRUSH,
+                                    GIMP_ICON_LOCK_CONTENT,
                                     _("Lock _pixels"),
                                     _("Lock position and _size"),
+                                    _("Lock visibility"),
                                     layer_name,
                                     layer_visible,
                                     layer_color_tag,
                                     layer_lock_content,
                                     layer_lock_position,
+                                    layer_lock_visibility,
                                     layer_options_dialog_callback,
                                     private);
 
@@ -190,9 +194,12 @@ layer_options_dialog_new (GimpImage                *image,
                     private);
 
   space_model =
-    gimp_enum_store_new_with_range (GIMP_TYPE_LAYER_COLOR_SPACE,
-                                    GIMP_LAYER_COLOR_SPACE_AUTO,
-                                    GIMP_LAYER_COLOR_SPACE_RGB_PERCEPTUAL);
+    gimp_enum_store_new_with_values (GIMP_TYPE_LAYER_COLOR_SPACE,
+                                     4,
+                                     GIMP_LAYER_COLOR_SPACE_AUTO,
+                                     GIMP_LAYER_COLOR_SPACE_RGB_LINEAR,
+                                     GIMP_LAYER_COLOR_SPACE_RGB_NON_LINEAR,
+                                     GIMP_LAYER_COLOR_SPACE_RGB_PERCEPTUAL);
 
   private->blend_space_combo = combo =
     gimp_enum_combo_box_new_with_model (GIMP_ENUM_STORE (space_model));
@@ -261,7 +268,7 @@ layer_options_dialog_new (GimpImage                *image,
       gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (spinbutton), TRUE);
       gtk_entry_set_width_chars (GTK_ENTRY (spinbutton), 10);
 
-      private->size_se = gimp_size_entry_new (1, GIMP_UNIT_PIXEL, "%a",
+      private->size_se = gimp_size_entry_new (1, gimp_unit_pixel (), "%a",
                                               TRUE, TRUE, FALSE, 10,
                                               GIMP_SIZE_ENTRY_UPDATE_SIZE);
 
@@ -274,7 +281,7 @@ layer_options_dialog_new (GimpImage                *image,
       gtk_widget_show (private->size_se);
 
       gimp_size_entry_set_unit (GIMP_SIZE_ENTRY (private->size_se),
-                                GIMP_UNIT_PIXEL);
+                                gimp_unit_pixel ());
 
       gimp_size_entry_set_resolution (GIMP_SIZE_ENTRY (private->size_se), 0,
                                       xres, FALSE);
@@ -318,7 +325,7 @@ layer_options_dialog_new (GimpImage                *image,
   gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (spinbutton), TRUE);
   gtk_entry_set_width_chars (GTK_ENTRY (spinbutton), 10);
 
-  private->offset_se = gimp_size_entry_new (1, GIMP_UNIT_PIXEL, "%a",
+  private->offset_se = gimp_size_entry_new (1, gimp_unit_pixel (), "%a",
                                             TRUE, TRUE, FALSE, 10,
                                             GIMP_SIZE_ENTRY_UPDATE_SIZE);
 
@@ -331,7 +338,7 @@ layer_options_dialog_new (GimpImage                *image,
   gtk_widget_show (private->offset_se);
 
   gimp_size_entry_set_unit (GIMP_SIZE_ENTRY (private->offset_se),
-                            GIMP_UNIT_PIXEL);
+                            gimp_unit_pixel ());
 
   gimp_size_entry_set_resolution (GIMP_SIZE_ENTRY (private->offset_se), 0,
                                   xres, FALSE);
@@ -391,7 +398,7 @@ layer_options_dialog_new (GimpImage                *image,
 
       filters = gimp_drawable_get_filters (GIMP_DRAWABLE (layer));
 
-      view = gimp_container_tree_view_new (filters, context,
+      view = gimp_container_list_view_new (filters, context,
                                            GIMP_VIEW_SIZE_SMALL, 0);
       gtk_container_add (GTK_CONTAINER (frame), view);
       gtk_widget_show (view);
@@ -411,7 +418,7 @@ layer_options_dialog_new (GimpImage                *image,
                           G_BINDING_INVERT_BOOLEAN);
 
   button = item_options_dialog_add_switch (dialog,
-                                           GIMP_ICON_TRANSPARENCY,
+                                           GIMP_ICON_LOCK_ALPHA,
                                            _("Lock _alpha"));
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button),
                                 private->lock_alpha);
@@ -458,6 +465,7 @@ layer_options_dialog_callback (GtkWidget    *dialog,
                                GimpColorTag  item_color_tag,
                                gboolean      item_lock_content,
                                gboolean      item_lock_position,
+                               gboolean      item_lock_visibility,
                                gpointer      user_data)
 {
   LayerOptionsDialog *private = user_data;
@@ -502,6 +510,7 @@ layer_options_dialog_callback (GtkWidget    *dialog,
                      item_color_tag,
                      item_lock_content,
                      item_lock_position,
+                     item_lock_visibility,
                      private->lock_alpha,
                      private->rename_text_layers,
                      private->user_data);
