@@ -25,6 +25,8 @@ GimpImage * file_open_image                 (Gimp                *gimp,
                                              GimpContext         *context,
                                              GimpProgress        *progress,
                                              GFile               *file,
+                                             gint                 vector_width,
+                                             gint                 vector_height,
                                              gboolean             as_new,
                                              GimpPlugInProcedure *file_proc,
                                              GimpRunMode          run_mode,
