@@ -14,7 +14,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifndef __GIMP_GEGL_ENUMS_H__
@@ -27,8 +27,8 @@ GType gimp_cage_mode_get_type (void) G_GNUC_CONST;
 
 typedef enum
 {
-  GIMP_CAGE_MODE_CAGE_CHANGE, /*< desc="Create or adjust the cage"           >*/
-  GIMP_CAGE_MODE_DEFORM       /*< desc="Deform the cage to deform the image" >*/
+  GIMP_CAGE_MODE_CAGE_CHANGE, /*< desc="Create or adjust the cage"            >*/
+  GIMP_CAGE_MODE_DEFORM       /*< desc="Deform the cage\nto deform the image" >*/
 } GimpCageMode;
 
 
