@@ -12,22 +12,23 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifndef __SESSION_H__
 #define __SESSION_H__
 
 
-void       session_init    (Gimp     *gimp);
-void       session_exit    (Gimp     *gimp);
+void       session_init    (Gimp       *gimp);
+void       session_exit    (Gimp       *gimp);
 
-void       session_restore (Gimp     *gimp);
-void       session_save    (Gimp     *gimp,
-                            gboolean  always_save);
+void       session_restore (Gimp       *gimp,
+                            GdkMonitor *monitor);
+void       session_save    (Gimp       *gimp,
+                            gboolean    always_save);
 
-gboolean   session_clear   (Gimp     *gimp,
-                            GError  **error);
+gboolean   session_clear   (Gimp       *gimp,
+                            GError    **error);
 
 
 #endif  /*  __SESSION_H__  */
