@@ -480,6 +480,8 @@ themes_apply_theme (Gimp          *gimp,
             "\n"
             "* { -GimpDock-tool-icon-size: %s; }"
             "\n"
+            "* { -GimpViewableDialog-tool-icon-size: %s; }"
+            "\n"
             "* { -GimpDockbook-tab-icon-size: %s; }"
             "\n"
             "* { -GimpColorNotebook-tab-icon-size: %s; }"
@@ -497,9 +499,9 @@ themes_apply_theme (Gimp          *gimp,
             "button, tab { padding: %dpx; }"
             "\n"
             "paned separator { padding: %dpx; }",
-            tool_icon_size, tool_icon_size, tab_icon_size, tab_icon_size,
-            tab_icon_size, button_icon_size, button_icon_size, tool_icon_size,
-            pal_padding, tab_padding, sep_padding);
+            tool_icon_size, tool_icon_size, tool_icon_size, tab_icon_size,
+            tab_icon_size, tab_icon_size, button_icon_size, button_icon_size,
+            tool_icon_size, pal_padding, tab_padding, sep_padding);
         }
 
       if (! error && config->font_relative_size != 1.0)
