@@ -12,12 +12,12 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
 
-#include <gio/gio.h>
+#include <gdk-pixbuf/gdk-pixbuf.h>
 
 #ifdef G_OS_WIN32
 #include <windows.h>
@@ -25,71 +25,136 @@
 
 #include "core/core-types.h"
 
-#include "file/file-utils.h"
+#ifndef GIMP_CONSOLE_COMPILATION
+/*  for the DBus service names  */
+#include "gui/gimpdbusservice.h"
+#endif
 
 #include "unique.h"
 
-
+#ifndef G_OS_WIN32
 static gboolean  gimp_unique_dbus_open  (const gchar **filenames,
-					 gboolean      as_new);
-#ifdef G_OS_WIN32
+                                         gboolean      as_new);
+static gboolean  gimp_unique_dbus_batch_run (const gchar  *batch_interpreter,
+                                             const gchar **batch_commands);
+#else
 static gboolean  gimp_unique_win32_open (const gchar **filenames,
-					 gboolean      as_new);
+                                         gboolean      as_new);
 #endif
 
+
 gboolean
 gimp_unique_open (const gchar **filenames,
-		  gboolean      as_new)
+                  gboolean      as_new)
 {
 #ifdef G_OS_WIN32
   return gimp_unique_win32_open (filenames, as_new);
+#elif defined (PLATFORM_OSX)
+  /* Opening files through "Open with" from other software is likely handled
+   * instead by gui_unique_quartz_init() by gtkosx signal handling.
+   *
+   * Opening files through command lines will always create new process, because
+   * dbus is usually not installed by default on macOS (and when it is, it may
+   * not work properly). See !808 and #8997.
+   */
+  return FALSE;
 #else
   return gimp_unique_dbus_open (filenames, as_new);
 #endif
 }
 
-#ifndef GIMP_CONSOLE_COMPILATION
-static gchar *
-gimp_unique_filename_to_uri (const gchar  *filename,
-			     const gchar  *cwd,
-			     GError      **error)
+gboolean
+gimp_unique_batch_run (const gchar  *batch_interpreter,
+                       const gchar **batch_commands)
 {
-  gchar *uri = NULL;
-
-  if (file_utils_filename_is_uri (filename, error))
-    {
-      uri = g_strdup (filename);
-    }
-  else if (! *error)
-    {
-      if (! g_path_is_absolute (filename))
-	{
-	  gchar *absolute = g_build_filename (cwd, filename, NULL);
-
-	  uri = g_filename_to_uri (absolute, NULL, error);
-
-	  g_free (absolute);
-	}
-      else
-	{
-	  uri = g_filename_to_uri (filename, NULL, error);
-	}
-    }
-
-  return uri;
+#ifdef G_OS_WIN32
+  g_printerr ("Batch commands cannot be run in existing instance in Win32.\n");
+  return FALSE;
+#elif defined (PLATFORM_OSX)
+  /* Running batch commands through command lines will always run in the new
+   * process, because dbus is usually not installed by default on macOS (and
+   * when it is, it may not work properly). See !808 and #8997.
+   */
+  return FALSE;
+#else
+  return gimp_unique_dbus_batch_run (batch_interpreter,
+                                     batch_commands);
+#endif
 }
+
+#ifdef G_OS_WIN32
+
+static gboolean
+gimp_unique_win32_open (const gchar **filenames,
+                        gboolean      as_new)
+{
+#ifndef GIMP_CONSOLE_COMPILATION
+
+/*  for the proxy window names  */
+#include "gui/gui-unique.h"
+
+  HWND  window_handle = FindWindowW (GIMP_UNIQUE_WIN32_WINDOW_CLASS,
+                                     GIMP_UNIQUE_WIN32_WINDOW_NAME);
+
+  if (window_handle)
+    {
+      COPYDATASTRUCT  copydata = { 0, };
+
+      if (filenames)
+        {
+          gchar  *cwd   = g_get_current_dir ();
+          gint    i;
+
+          for (i = 0; filenames[i]; i++)
+            {
+              GFile *file;
+              file = g_file_new_for_commandline_arg_and_cwd (filenames[i], cwd);
+
+              if (file)
+                {
+                  gchar *uri = g_file_get_uri (file);
+
+                  copydata.lpData = uri;
+                  copydata.cbData = strlen (uri) + 1;  /* size in bytes   */
+                  copydata.dwData = (long) as_new;
+
+                  SendMessage (window_handle,
+                               WM_COPYDATA, (WPARAM) window_handle, (LPARAM) &copydata);
+
+                  g_free (uri);
+                  g_object_unref (file);
+                }
+              else
+                {
+                  g_printerr ("conversion to uri failed for '%s'\n",
+                              filenames[i]);
+                }
+            }
+
+          g_free (cwd);
+        }
+      else
+        {
+          SendMessage (window_handle,
+                       WM_COPYDATA, (WPARAM) window_handle, (LPARAM) &copydata);
+        }
+
+      return TRUE;
+    }
+
 #endif
 
+  return FALSE;
+}
+
+#else
 
 static gboolean
 gimp_unique_dbus_open (const gchar **filenames,
-		       gboolean      as_new)
+                       gboolean      as_new)
 {
 #ifndef GIMP_CONSOLE_COMPILATION
 
-/*  for the DBus service names  */
-#include "gui/gimpdbusservice.h"
-
   GDBusConnection *connection;
   GError          *error = NULL;
 
@@ -107,18 +172,19 @@ gimp_unique_dbus_open (const gchar **filenames,
 
           for (i = 0; filenames[i] && success; i++)
             {
-              GError *error = NULL;
-	      gchar  *uri   = gimp_unique_filename_to_uri (filenames[i],
-                                                           cwd, &error);
+              GFile *file;
 
-              if (uri)
+              file = g_file_new_for_commandline_arg_and_cwd (filenames[i], cwd);
+
+              if (file)
                 {
                   GVariant *result;
+                  gchar    *uri = g_file_get_uri (file);
 
                   result = g_dbus_connection_call_sync (connection,
                                                         GIMP_DBUS_SERVICE_NAME,
                                                         GIMP_DBUS_SERVICE_PATH,
-                                                        GIMP_DBUS_SERVICE_INTERFACE,
+                                                        GIMP_DBUS_INTERFACE_NAME,
                                                         method,
                                                         g_variant_new ("(s)",
                                                                        uri),
@@ -126,17 +192,20 @@ gimp_unique_dbus_open (const gchar **filenames,
                                                         G_DBUS_CALL_FLAGS_NO_AUTO_START,
                                                         -1,
                                                         NULL, NULL);
+
+                  g_free (uri);
+
                   if (result)
                     g_variant_unref (result);
                   else
                     success = FALSE;
 
-                  g_free (uri);
+                  g_object_unref (file);
                 }
               else
                 {
-                  g_printerr ("conversion to uri failed: %s\n", error->message);
-                  g_clear_error (&error);
+                  g_printerr ("conversion to uri failed for '%s'\n",
+                              filenames[i]);
                 }
             }
 
@@ -149,7 +218,7 @@ gimp_unique_dbus_open (const gchar **filenames,
           result = g_dbus_connection_call_sync (connection,
                                                 GIMP_DBUS_SERVICE_NAME,
                                                 GIMP_DBUS_SERVICE_PATH,
-                                                GIMP_DBUS_SERVICE_INTERFACE,
+                                                GIMP_DBUS_INTERFACE_NAME,
                                                 "Activate",
                                                 NULL,
                                                 NULL,
@@ -176,66 +245,63 @@ gimp_unique_dbus_open (const gchar **filenames,
   return FALSE;
 }
 
-#ifdef G_OS_WIN32
 
 static gboolean
-gimp_unique_win32_open (const gchar **filenames,
-			gboolean      as_new)
+gimp_unique_dbus_batch_run (const gchar  *batch_interpreter,
+                            const gchar **batch_commands)
 {
 #ifndef GIMP_CONSOLE_COMPILATION
 
-/*  for the proxy window names  */
-#include "gui/gui-unique.h"
+  GDBusConnection *connection;
+  GError          *error = NULL;
 
-  HWND  window_handle = FindWindowW (GIMP_UNIQUE_WIN32_WINDOW_CLASS,
-				     GIMP_UNIQUE_WIN32_WINDOW_NAME);
+  connection = g_bus_get_sync (G_BUS_TYPE_SESSION, NULL, &error);
 
-  if (window_handle)
+  if (connection)
     {
-      COPYDATASTRUCT  copydata = { 0, };
+      const gchar *method = "BatchRun";
+      gboolean     success = TRUE;
+      gint         i;
 
-      if (filenames)
+      for (i = 0; batch_commands[i] && success; i++)
         {
-          gchar  *cwd   = g_get_current_dir ();
-          GError *error = NULL;
-          gint    i;
+          GVariant    *result;
+          const gchar *interpreter;
 
-          for (i = 0; filenames[i]; i++)
-            {
-              gchar *uri;
+          /* NULL is not a valid string GVariant. */
+          interpreter = batch_interpreter ? batch_interpreter : "";
 
-              uri = gimp_unique_filename_to_uri (filenames[i], cwd, &error);
+          result = g_dbus_connection_call_sync (connection,
+                                                GIMP_DBUS_SERVICE_NAME,
+                                                GIMP_DBUS_SERVICE_PATH,
+                                                GIMP_DBUS_INTERFACE_NAME,
+                                                method,
+                                                g_variant_new ("(ss)",
+                                                               interpreter,
+                                                               batch_commands[i]),
+                                                NULL,
+                                                G_DBUS_CALL_FLAGS_NO_AUTO_START,
+                                                -1,
+                                                NULL, NULL);
 
-              if (uri)
-                {
-                  copydata.lpData = uri;
-                  copydata.cbData = strlen (uri) + 1;  /* size in bytes   */
-                  copydata.dwData = (long) as_new;
 
-                  SendMessage (window_handle,
-                               WM_COPYDATA, (WPARAM) window_handle, (LPARAM) &copydata);
-                }
-              else
-                {
-                  g_printerr ("conversion to uri failed: %s\n", error->message);
-                  g_clear_error (&error);
-                }
-            }
-
-          g_free (cwd);
-        }
-      else
-        {
-          SendMessage (window_handle,
-                       WM_COPYDATA, (WPARAM) window_handle, (LPARAM) &copydata);
+          if (result)
+            g_variant_unref (result);
+          else
+            success = FALSE;
         }
 
-      return TRUE;
+      g_object_unref (connection);
+
+      return success;
+    }
+  else
+    {
+      g_printerr ("%s\n", error->message);
+      g_clear_error (&error);
     }
-
 #endif
 
   return FALSE;
 }
-
 #endif  /* G_OS_WIN32 */
