@@ -386,6 +386,8 @@ gimp_file_dialog_dispose (GObject *object)
 {
   GimpFileDialog *dialog = GIMP_FILE_DIALOG (object);
 
+  gimp_widget_free_native_handle (GTK_WIDGET (dialog), &dialog->window_handle);
+
   G_OBJECT_CLASS (parent_class)->dispose (object);
 
   dialog->progress = NULL;
@@ -395,8 +397,6 @@ gimp_file_dialog_dispose (GObject *object)
   g_clear_pointer (&dialog->automatic_help_id, g_free);
   g_clear_pointer (&dialog->automatic_label,   g_free);
   g_clear_pointer (&dialog->file_filter_label, g_free);
-
-  gimp_widget_free_native_handle (GTK_WIDGET (dialog), &dialog->window_handle);
 }
 
 static gboolean
@@ -646,7 +646,10 @@ gimp_file_dialog_set_sensitive (GimpFileDialog *dialog,
       if (! g_list_next (list))
         break;
 
-      gtk_widget_set_sensitive (list->data, sensitive);
+      /* Don't freeze GimpProgressBox, otherwise we won't see the
+       * progress bar's movement when loading images */
+      if (! GIMP_IS_PROGRESS_BOX (list->data))
+        gtk_widget_set_sensitive (list->data, sensitive);
     }
 
   g_list_free (children);
