@@ -8,9 +8,14 @@ if procedure is None:
   sys.exit(os.EX_OK)
 config    = procedure.create_config()
 
-def export_scaled_img(image, target_width, target_height, export_path):
-  img        = image.duplicate()
-  layer = img.flatten()
+
+def export_scaled_img(image, target_height, export_path):
+  img    = image.duplicate()
+  img.undo_disable()
+  layer  = img.flatten()
+  width  = layer.get_width()
+  height = layer.get_height()
+  target_width  = int(target_height / height * width)
   layer.scale(target_width, target_height, False)
   img.resize(target_width, target_height, 0, 0)
 
@@ -18,14 +23,18 @@ def export_scaled_img(image, target_width, target_height, export_path):
   config.set_property("file", Gio.file_new_for_path(export_path))
   retval = procedure.run(config)
   if retval.index(0) != Gimp.PDBStatusType.SUCCESS:
-    sys.exit(os.EX_SOFTWARE)
+    sys.exit(70)
   img.delete()
 
 
 # See https://gitlab.gnome.org/GNOME/gimp-data/-/issues/5
-def export_blurred_img(image, target_width, target_height, export_path):
-  img        = image.duplicate()
-  layer = img.flatten()
+def export_blurred_img(image, target_height, export_path):
+  img    = image.duplicate()
+  img.undo_disable()
+  layer  = img.flatten()
+  width  = layer.get_width()
+  height = layer.get_height()
+  target_width  = int(target_height / height * width)
   layer.scale(target_width, target_height, False)
   img.resize(target_width, target_height, 0, 0)
 
@@ -46,12 +55,13 @@ def export_blurred_img(image, target_width, target_height, export_path):
   config.set_property("file", Gio.file_new_for_path(export_path))
   retval = procedure.run(config)
   if retval.index(0) != Gimp.PDBStatusType.SUCCESS:
-    sys.exit(os.EX_SOFTWARE)
+    sys.exit(70)
   img.delete()
 
 
 def export_cropped_img(image, target_width, target_height, export_path):
   img        = image.duplicate()
+  img.undo_disable()
   w          = img.get_width()
   h          = img.get_height()
   img.flatten()
@@ -68,23 +78,30 @@ def export_cropped_img(image, target_width, target_height, export_path):
   config.set_property("file", Gio.file_new_for_path(export_path))
   retval = procedure.run(config)
   if retval.index(0) != Gimp.PDBStatusType.SUCCESS:
-    sys.exit(os.EX_SOFTWARE)
+    sys.exit(70)
   img.delete()
 
 
-# https://gitlab.gnome.org/GNOME/gimp/-/issues/11677 
-export_scaled_img(image, 558, 314, 'build/windows/installer/installsplash_top.scale-100.bmp')
-export_scaled_img(image, 686, 386, 'build/windows/installer/installsplash_top.scale-125.bmp')
-export_scaled_img(image, 816, 459, 'build/windows/installer/installsplash_top.scale-150.bmp')
-export_scaled_img(image, 988, 556, 'build/windows/installer/installsplash_top.scale-175.bmp')
-export_scaled_img(image, 1074, 604, 'build/windows/installer/installsplash_top.scale-200.bmp')
-export_scaled_img(image, 1244, 700, 'build/windows/installer/installsplash_top.scale-225.bmp')
-export_scaled_img(image, 1417, 797, 'build/windows/installer/installsplash_top.scale-250.bmp')
-export_blurred_img(image, 558, 314, 'build/windows/installer/installsplash_bottom.bmp')
+image.undo_disable()
+
+# Safe (taller than necessary) heights derived from: https://jrsoftware.org/ishelp/index.php?topic=setup_wizardimagefile
+export_scaled_img(image, 314, 'build/windows/installer/installsplash_top.scale-100.bmp')
+export_scaled_img(image, 386, 'build/windows/installer/installsplash_top.scale-125.bmp')
+export_scaled_img(image, 459, 'build/windows/installer/installsplash_top.scale-150.bmp')
+export_scaled_img(image, 556, 'build/windows/installer/installsplash_top.scale-175.bmp')
+export_scaled_img(image, 604, 'build/windows/installer/installsplash_top.scale-200.bmp')
+export_scaled_img(image, 700, 'build/windows/installer/installsplash_top.scale-225.bmp')
+export_scaled_img(image, 797, 'build/windows/installer/installsplash_top.scale-250.bmp')
+export_blurred_img(image, 314, 'build/windows/installer/installsplash_bottom.bmp')
 
 # Avoid the images being re-generated at each build.
 pathlib.Path('gimp-data/images/stamp-installsplash.bmp').touch()
 
+# Needed by the installer .iss script to maintain the splash aspect ratio: https://gitlab.gnome.org/GNOME/gimp/-/issues/11677 
+with open('build/windows/installer/splash-dimensions.h', 'w') as f:
+  f.write('#define GIMP_SPLASH_WIDTH  {}\n'.format(image.get_width()))
+  f.write('#define GIMP_SPLASH_HEIGHT {}\n'.format(image.get_height()))
+
 
 # https://jrsoftware.org/ishelp/index.php?topic=setup_wizardimagefile
 export_cropped_img(image, 164, 314, 'build/windows/installer/install-end.scale-100.bmp')
