@@ -200,6 +200,15 @@ gimp_main (GType  plug_in_type,
 
   gint i, j, k;
 
+  /* Make plugins output available on console */
+  if (AttachConsole (ATTACH_PARENT_PROCESS) != 0 && ! g_getenv ("TERM") && ! g_getenv ("SHELL"))
+    {
+      /* 'r' is needed to prevent interleaving and '+' to support colors */
+      freopen ("CONOUT$", "r+", stdout);
+      freopen ("CONOUT$", "r+", stderr);
+      _flushall ();
+    }
+
   /* Reduce risks */
   SetDllDirectoryW (L"");
 
@@ -1145,11 +1154,6 @@ _gimp_config (GPConfig *config)
                 "application-license", "GPL3",
                 NULL);
 
-  /* XXX Running gegl_init() before gegl_config() is not appreciated by
-   * GEGL and generates a bunch of CRITICALs.
-   */
-  babl_init ();
-
   g_clear_object (&_check_custom_color1);
   _check_custom_color1 = gegl_color_new (NULL);
   pixel = g_bytes_get_data (config->check_custom_color1, &bpp);
