@@ -21,7 +21,18 @@
 #include "config.h"
 
 #include <string.h>
+#ifdef HAVE_SYS_TIME_H
 #include <sys/time.h>
+#endif
+#if defined(_WIN32) && !defined(HAVE_SYS_TIME_H)
+#include <time.h>
+#include <windows.h>
+#if defined(_MSC_VER) || defined(_MSC_EXTENSIONS)
+#define DELTA_EPOCH_IN_MICROSECS 11644473600000000Ui64
+#else
+#define DELTA_EPOCH_IN_MICROSECS 11644473600000000ULL
+#endif
+#endif
 
 #include <gexiv2/gexiv2.h>
 
@@ -57,23 +68,31 @@ static GList*     gimp_image_metadata_set_xmp_structs       (GList
  * gimp_image_metadata_save_prepare:
  * @image:           The original image
  * @mime_type:       The saved file's mime-type
- * @suggested_flags: Suggested default values for the @flags passed to
- *                   gimp_image_metadata_save_finish()
+ * @suggested_flags: Suggested default values for the metadata to export.
  *
- * Gets the image metadata for saving it using
- * gimp_image_metadata_save_finish().
+ * Gets the image metadata for storing it in an exported file.
  *
- * The @suggested_flags are determined from what kind of metadata
- * (Exif, XMP, ...) is actually present in the image and the preferences
- * for metadata exporting.
- * The calling application may still update @available_flags, for
+ * *Note: There is normally no need to call this function because it's
+ * already called by [class@ExportProcedure] at the start and the
+ * metadata is passed to the `run()` callback.*
+ *
+ * *You may call it separately for instance if you set @export_metadata
+ * to %NULL in [ctor@Gimp.ExportProcedure.new] to prevent `libgimp`
+ * from trying to store the metadata in the exported file, yet you wish
+ * to process and store the metadata yourself using custom API.*
+ *
+ * The @suggested_flags are determined from what kind of metadata (Exif,
+ * XMP, ...) is actually present in the image and the preferences for
+ * metadata exporting.
+ * The calling application may still ignore @suggested_flags, for
  * instance to follow the settings from a previous export in the same
  * session, or a previous export of the same image. But it should not
  * override the preferences without a good reason since it is a data
  * leak.
  *
- * The suggested value for %GIMP_METADATA_SAVE_THUMBNAIL is determined by
- * whether there was a thumbnail in the previously imported image.
+ * The suggested value for [flags@Gimp.MetadataSaveFlags.THUMBNAIL] is
+ * determined by whether there was a thumbnail in the previously
+ * imported image.
  *
  * Returns: (transfer full): The image's metadata, prepared for saving.
  *
@@ -98,8 +117,6 @@ gimp_image_metadata_save_prepare (GimpImage             *image,
   if (metadata)
     {
       GDateTime      *datetime;
-      GimpParasite   *comment_parasite;
-      gchar          *comment = NULL;
       gint            image_width;
       gint            image_height;
       gdouble         xres;
@@ -113,63 +130,30 @@ gimp_image_metadata_save_prepare (GimpImage             *image,
 
       datetime = g_date_time_new_now_local ();
 
-      comment_parasite = gimp_image_get_parasite (image, "gimp-comment");
-      if (comment_parasite)
-        {
-          guint32  parasite_size;
-
-          comment = (gchar *) gimp_parasite_get_data (comment_parasite, &parasite_size);
-          comment = g_strndup (comment, parasite_size);
-
-          gimp_parasite_free (comment_parasite);
-        }
-
       /* Exif */
 
       if (! gimp_export_exif () ||
           ! gexiv2_metadata_has_exif (g2metadata))
         *suggested_flags &= ~GIMP_METADATA_SAVE_EXIF;
 
-      if (comment)
-        {
-          gexiv2_metadata_try_set_tag_string (g2metadata,
-                                              "Exif.Photo.UserComment",
-                                              comment, &error);
-          if (error)
-            {
-              g_warning ("%s: failed to set metadata '%s': %s\n",
-                         G_STRFUNC, "Exif.Photo.UserComment", error->message);
-              g_clear_error (&error);
-            }
-
-          gexiv2_metadata_try_set_tag_string (g2metadata,
-                                              "Exif.Image.ImageDescription",
-                                              comment, &error);
-          if (error)
-            {
-              g_warning ("%s: failed to set metadata '%s': %s\n",
-                         G_STRFUNC, "Exif.Image.ImageDescription", error->message);
-              g_clear_error (&error);
-            }
-        }
-
-      g_snprintf (buffer, sizeof (buffer),
-                  "%d:%02d:%02d %02d:%02d:%02d",
-                  g_date_time_get_year (datetime),
-                  g_date_time_get_month (datetime),
-                  g_date_time_get_day_of_month (datetime),
-                  g_date_time_get_hour (datetime),
-                  g_date_time_get_minute (datetime),
-                  g_date_time_get_second (datetime));
+      datetime_buf = g_date_time_format (datetime, "%Y:%m:%d %T");
       gexiv2_metadata_try_set_tag_string (g2metadata,
                                           "Exif.Image.DateTime",
-                                          buffer, &error);
+                                          datetime_buf, &error);
       if (error)
         {
           g_warning ("%s: failed to set metadata '%s': %s\n",
                      G_STRFUNC, "Exif.Image.DateTime", error->message);
           g_clear_error (&error);
         }
+      g_clear_pointer (&datetime_buf, g_free);
+
+      /* Timezone is separate */
+      datetime_buf = g_date_time_format (datetime, "\%:z");
+      gexiv2_metadata_try_set_tag_string (g2metadata,
+                                          "Exif.Photo.OffsetTime",
+                                          datetime_buf, NULL);
+      g_clear_pointer (&datetime_buf, g_free);
 
       gexiv2_metadata_try_set_tag_string (g2metadata,
                                           "Exif.Image.Software",
@@ -205,7 +189,7 @@ gimp_image_metadata_save_prepare (GimpImage             *image,
         }
 
       /* XMP uses datetime in ISO 8601 format */
-      datetime_buf = g_date_time_format (datetime, "%Y:%m:%dT%T\%:z");
+      datetime_buf = g_date_time_format (datetime, "%Y-%m-%dT%T\%:z");
 
       gexiv2_metadata_try_set_tag_string (g2metadata,
                                           "Xmp.xmp.ModifyDate",
@@ -271,7 +255,6 @@ gimp_image_metadata_save_prepare (GimpImage             *image,
 
       g_free (datetime_buf);
       g_date_time_unref (datetime);
-      g_clear_pointer (&comment, g_free);
 
       /* EXIF Thumbnail */
 
@@ -330,7 +313,11 @@ gimp_image_metadata_save_prepare (GimpImage             *image,
 static const gchar *
 gimp_fix_xmp_tag (const gchar *tag)
 {
-  gchar *substring;
+  const gchar org[] = "/Iptc4xmpExt";
+  const gchar new[] = "/iptcExt";
+  gchar *substring, *tail;
+
+  G_STATIC_ASSERT (sizeof new < sizeof org);
 
   /* Due to problems using /Iptc4xmpExt namespace (/iptcExt is used
    * instead by Exiv2) we replace all occurrences with /iptcExt which
@@ -338,28 +325,21 @@ gimp_fix_xmp_tag (const gchar *tag)
    * metadata to fail. This has to be done after getting the values
    * from the source metadata since that source uses the original
    * tag names and would otherwise return NULL as value.
-   * /Iptc4xmpExt length = 12
-   * /iptcExt     length =  8
+   * Multiple occurrences are possible: e.g.:
+   * Xmp.iptcExt.ImageRegion[3]/Iptc4xmpExt:RegionBoundary/Iptc4xmpExt:rbVertices[1]/Iptc4xmpExt:rbX
    */
 
-  substring = strstr (tag, "/Iptc4xmpExt");
-  while (substring)
+  while ((substring = strstr (tag, org)))
     {
-      gint len_tag = strlen (tag);
-      gint len_end;
+      strcpy (substring, new);
+
+      tail = substring + strlen (org);
+      substring += strlen (new);
+
+      while ((*substring++ = *tail++))
+        ;
 
-      len_end = len_tag - (substring - tag) - 12;
-      strncpy (substring, "/iptcExt", 8);
-      substring += 8;
-      /* Using memmove: we have overlapping source and dest */
-      memmove (substring, substring+4, len_end);
-      substring[len_end] = '\0';
       g_debug ("Fixed tag value: %s", tag);
-
-      /* Multiple occurrences are possible: e.g.:
-       * Xmp.iptcExt.ImageRegion[3]/Iptc4xmpExt:RegionBoundary/Iptc4xmpExt:rbVertices[1]/Iptc4xmpExt:rbX
-       */
-      substring = strstr (tag, "/Iptc4xmpExt");
     }
   return tag;
 }
@@ -612,7 +592,6 @@ gimp_image_metadata_set_xmp_structs (GList          *xmp_list,
                                    G_STRFUNC, second_struct, error->message);
                       g_clear_error (&error);
                     }
-                  g_free (second_struct);
                 }
               else
                 {
@@ -623,6 +602,7 @@ gimp_image_metadata_set_xmp_structs (GList          *xmp_list,
                       exclude = g_list_prepend (exclude, g_strdup (tag_split[1]));
                     }
                 }
+              g_free (second_struct);
             }
         }
 
@@ -634,6 +614,50 @@ gimp_image_metadata_set_xmp_structs (GList          *xmp_list,
   return exclude;
 }
 
+#if defined(_WIN32) && ! defined(HAVE_SYS_TIME_H)
+struct timezone
+{
+  int tz_minuteswest; /* minutes W of Greenwich */
+  int tz_dsttime;     /* type of dst correction */
+};
+
+int
+gettimeofday (struct timeval *tv, struct timezone *tz)
+{
+  FILETIME         ft;
+  unsigned __int64 tmpres = 0;
+  static int       tzflag = 0;
+
+  if (NULL != tv)
+    {
+      GetSystemTimeAsFileTime (&ft);
+
+      tmpres |= ft.dwHighDateTime;
+      tmpres <<= 32;
+      tmpres |= ft.dwLowDateTime;
+
+      tmpres /= 10; /*convert into microseconds*/
+      /*converting file time to unix epoch*/
+      tmpres -= DELTA_EPOCH_IN_MICROSECS;
+      tv->tv_sec  = (long) (tmpres / 1000000UL);
+      tv->tv_usec = (long) (tmpres % 1000000UL);
+    }
+
+  if (NULL != tz)
+    {
+      if (! tzflag)
+        {
+          _tzset ();
+          tzflag++;
+        }
+      tz->tz_minuteswest = _timezone / 60;
+      tz->tz_dsttime     = _daylight;
+    }
+
+  return 0;
+}
+#endif
+
 /**
  * gimp_image_metadata_save_filter:
  * @image:     The actually saved image
@@ -644,19 +668,24 @@ gimp_image_metadata_set_xmp_structs (GList          *xmp_list,
  * @error:     Return location for error message
  *
  * Filters the @metadata retrieved from the image with
- * gimp_image_metadata_save_prepare(),
- * taking into account the passed @flags.
+ * [method@Gimp.Image.metadata_save_prepare], taking into account the
+ * passed @flags.
+ *
+ * *Note: There is normally no need to call this function because it's
+ * already called by [class@ExportProcedure] after the `run()`
+ * callback.*
  *
  * Note that the @image passed to this function might be different
- * from the image passed to gimp_image_metadata_save_prepare(), due
+ * from the image passed to `gimp_image_metadata_save_prepare()`, due
  * to whatever file export conversion happened in the meantime
  *
- * This is an alternative to gimp_image_metadata_save_finish when you
- * want to save metadata yourself and you need only filtering processing.
+ * This can be used as an alternative to core metadata handling when you
+ * want to save metadata yourself and you need only filtering
+ * processing.
  *
- * Returns: (transfer full): Filtered metadata or NULL in case of failure.
- *
- * Use g_object_unref() when returned metadata are no longer needed
+ * Returns: (transfer full): Filtered metadata or %NULL in case of failure.
+ *          Use [GObject.Object.unref] when returned metadata are no
+ *          longer needed
  *
  * Since: 3.0
  */
@@ -712,7 +741,51 @@ gimp_image_metadata_save_filter (GimpImage            *image,
 
   if ((flags & GIMP_METADATA_SAVE_EXIF) && support_exif)
     {
-      gchar **exif_data = gexiv2_metadata_get_exif_tags (GEXIV2_METADATA (metadata));
+      gchar **exif_data = NULL;
+
+      if ((flags & GIMP_METADATA_SAVE_COMMENT))
+        {
+          GimpParasite *comment_parasite;
+
+          comment_parasite = gimp_image_get_parasite (image, "gimp-comment");
+          if (comment_parasite)
+            {
+              guint32  parasite_size;
+              gchar   *comment = NULL;
+
+              comment = (gchar *) gimp_parasite_get_data (comment_parasite, &parasite_size);
+              comment = g_strndup (comment, parasite_size);
+
+              gimp_parasite_free (comment_parasite);
+
+              if (comment)
+                {
+                  gexiv2_metadata_try_set_tag_string (GEXIV2_METADATA (metadata),
+                                                      "Exif.Photo.UserComment",
+                                                      comment, &code_error);
+                  if (code_error)
+                    {
+                      g_warning ("%s: failed to set metadata '%s': %s\n",
+                                 G_STRFUNC, "Exif.Photo.UserComment", code_error->message);
+                      g_clear_error (&code_error);
+                    }
+
+                  gexiv2_metadata_try_set_tag_string (GEXIV2_METADATA (metadata),
+                                                      "Exif.Image.ImageDescription",
+                                                      comment, &code_error);
+                  if (code_error)
+                    {
+                      g_warning ("%s: failed to set metadata '%s': %s\n",
+                                 G_STRFUNC, "Exif.Image.ImageDescription", code_error->message);
+                      g_clear_error (&code_error);
+                    }
+
+                  g_free (comment);
+                }
+            }
+        }
+
+      exif_data = gexiv2_metadata_get_exif_tags (GEXIV2_METADATA (metadata));
 
       for (i = 0; exif_data[i] != NULL; i++)
         {
@@ -861,6 +934,19 @@ gimp_image_metadata_save_filter (GimpImage            *image,
       g_strfreev (iptc_data);
     }
 
+  if (support_exif)
+    {
+      /* Remove all old thumbnail metadata: third-party software may have
+       * written tags we don't expect. */
+      gexiv2_metadata_try_erase_exif_thumbnail (new_g2metadata, &code_error);
+      if (code_error)
+        {
+          g_warning ("%s: failed to erase EXIF thumbnail: %s",
+                     G_STRFUNC, code_error->message);
+          g_clear_error (&code_error);
+        }
+    }
+
   if (flags & GIMP_METADATA_SAVE_THUMBNAIL && support_exif)
     {
       GdkPixbuf *thumb_pixbuf;
@@ -887,6 +973,11 @@ gimp_image_metadata_save_filter (GimpImage            *image,
           thumbw = EXIF_THUMBNAIL_SIZE * image_width / image_height;
         }
 
+      /* TODO: currently our exported thumbnails are always RGB because
+       * GdkPixbuf only supports RGB so far. While RGB thumbnail may make sense,
+       * even for CMYK images or other color spaces, grayscale images would be
+       * better served with a grayscale thumbnail too.
+       */
       thumb_pixbuf = gimp_image_get_thumbnail (image, thumbw, thumbh,
                                                GIMP_PIXBUF_KEEP_ALPHA);
 
@@ -975,17 +1066,6 @@ gimp_image_metadata_save_filter (GimpImage            *image,
 
       g_object_unref (thumb_pixbuf);
     }
-  else
-    {
-      /* Remove Thumbnail */
-      gexiv2_metadata_try_erase_exif_thumbnail (new_g2metadata, &code_error);
-      if (code_error)
-        {
-          g_warning ("%s: failed to erase EXIF thumbnail: %s\n",
-                     G_STRFUNC, code_error->message);
-          g_clear_error (&code_error);
-        }
-    }
 
   if (flags & GIMP_METADATA_SAVE_COLOR_PROFILE)
     {
@@ -1003,8 +1083,11 @@ gimp_image_metadata_save_filter (GimpImage            *image,
   return new_metadata;
 }
 
+
+/* Internal functions */
+
 /**
- * gimp_image_metadata_save_finish:
+ * _gimp_image_metadata_save_finish:
  * @image:     The actually saved image
  * @mime_type: The saved file's mime-type
  * @metadata:  The metadata to write to @file
@@ -1012,25 +1095,28 @@ gimp_image_metadata_save_filter (GimpImage            *image,
  * @file:      The file @image was saved to
  * @error:     Return location for error message
  *
+ * *Note: There is normally no need to call this function because it's
+ * already called by [class@ExportProcedure] at the end of the `run()` callback.*
+ *
  * Saves the @metadata retrieved from the image with
- * gimp_image_metadata_save_prepare() to @file, taking into account
- * the passed @flags.
+ * [method@Gimp.Image.metadata_save_prepare] to @file, taking into
+ * account the passed @flags.
  *
  * Note that the @image passed to this function might be different
- * from the image passed to gimp_image_metadata_save_prepare(), due
- * to whatever file export conversion happened in the meantime
+ * from the image passed to `gimp_image_metadata_save_prepare()`, due to
+ * whatever file export conversion happened in the meantime
  *
  * Returns: Whether the save was successful.
  *
  * Since: 2.10
  */
 gboolean
-gimp_image_metadata_save_finish (GimpImage            *image,
-                                 const gchar          *mime_type,
-                                 GimpMetadata         *metadata,
-                                 GimpMetadataSaveFlags flags,
-                                 GFile                *file,
-                                 GError              **error)
+_gimp_image_metadata_save_finish (GimpImage            *image,
+                                  const gchar          *mime_type,
+                                  GimpMetadata         *metadata,
+                                  GimpMetadataSaveFlags flags,
+                                  GFile                *file,
+                                  GError              **error)
 {
   GimpMetadata *new_metadata;
   gboolean      success = FALSE;
