@@ -25,7 +25,6 @@
 #include "libgimpbase/gimpprotocol.h"
 #include "libgimpbase/gimpwire.h"
 
-#include "gimp-private.h"
 #include "gimpgpparams.h"
 #include "gimppdb-private.h"
 #include "gimppdb_pdb.h"
@@ -42,8 +41,10 @@
  */
 
 
-struct _GimpPDBPrivate
+struct _GimpPDB
 {
+  GObject             parent_instance;
+
   GimpPlugIn         *plug_in;
 
   GHashTable         *procedures;
@@ -60,7 +61,7 @@ static void   gimp_pdb_set_error (GimpPDB        *pdb,
                                   GimpValueArray *return_values);
 
 
-G_DEFINE_TYPE_WITH_PRIVATE (GimpPDB, gimp_pdb, G_TYPE_OBJECT)
+G_DEFINE_TYPE (GimpPDB, gimp_pdb, G_TYPE_OBJECT)
 
 #define parent_class gimp_pdb_parent_class
 
@@ -77,12 +78,10 @@ gimp_pdb_class_init (GimpPDBClass *klass)
 static void
 gimp_pdb_init (GimpPDB *pdb)
 {
-  pdb->priv = gimp_pdb_get_instance_private (pdb);
+  pdb->procedures = g_hash_table_new_full (g_str_hash, g_str_equal,
+                                           g_free, g_object_unref);
 
-  pdb->priv->procedures = g_hash_table_new_full (g_str_hash, g_str_equal,
-                                                 g_free, g_object_unref);
-
-  pdb->priv->error_status = GIMP_PDB_SUCCESS;
+  pdb->error_status = GIMP_PDB_SUCCESS;
 }
 
 static void
@@ -90,7 +89,7 @@ gimp_pdb_dispose (GObject *object)
 {
   GimpPDB *pdb = GIMP_PDB (object);
 
-  g_clear_pointer (&pdb->priv->procedures, g_hash_table_unref);
+  g_clear_pointer (&pdb->procedures, g_hash_table_unref);
 
   G_OBJECT_CLASS (parent_class)->dispose (object);
 }
@@ -100,8 +99,8 @@ gimp_pdb_finalize (GObject *object)
 {
   GimpPDB *pdb = GIMP_PDB (object);
 
-  g_clear_object (&pdb->priv->plug_in);
-  g_clear_pointer (&pdb->priv->error_message, g_free);
+  g_clear_object (&pdb->plug_in);
+  g_clear_pointer (&pdb->error_message, g_free);
 
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
@@ -115,7 +114,7 @@ _gimp_pdb_new (GimpPlugIn *plug_in)
 
   pdb = g_object_new (GIMP_TYPE_PDB, NULL);
 
-  pdb->priv->plug_in = g_object_ref (plug_in);
+  pdb->plug_in = g_object_ref (plug_in);
 
   return pdb;
 }
@@ -125,7 +124,7 @@ _gimp_pdb_get_plug_in (GimpPDB *pdb)
 {
   g_return_val_if_fail (GIMP_IS_PDB (pdb), NULL);
 
-  return pdb->priv->plug_in;
+  return pdb->plug_in;
 }
 
 /**
@@ -173,235 +172,21 @@ gimp_pdb_lookup_procedure (GimpPDB     *pdb,
   g_return_val_if_fail (GIMP_IS_PDB (pdb), NULL);
   g_return_val_if_fail (gimp_is_canonical_identifier (procedure_name), NULL);
 
-  procedure = g_hash_table_lookup (pdb->priv->procedures, procedure_name);
+  procedure = g_hash_table_lookup (pdb->procedures, procedure_name);
 
-  if (! procedure)
+  if (! procedure && gimp_is_canonical_identifier (procedure_name) &&
+      _gimp_pdb_proc_exists (procedure_name))
     {
       procedure = _gimp_pdb_procedure_new (pdb, procedure_name);
 
       if (procedure)
-        g_hash_table_insert (pdb->priv->procedures,
+        g_hash_table_insert (pdb->procedures,
                              g_strdup (procedure_name), procedure);
     }
 
   return procedure;
 }
 
-/**
- * gimp_pdb_run_procedure: (skip)
- * @pdb:            the #GimpPDB object.
- * @procedure_name: the procedure registered name.
- * @first_type:     the #GType of the first argument, or #G_TYPE_NONE.
- * @...:            the call arguments.
- *
- * Runs the procedure named @procedure_name with arguments given as
- * list of (#GType, value) pairs, terminated by #G_TYPE_NONE.
- *
- * Returns: (transfer full): the return values for the procedure call.
- *
- * Since: 3.0
- */
-GimpValueArray *
-gimp_pdb_run_procedure (GimpPDB     *pdb,
-                        const gchar *procedure_name,
-                        GType        first_type,
-                        ...)
-{
-  GimpValueArray *return_values;
-  va_list         args;
-
-  g_return_val_if_fail (GIMP_IS_PDB (pdb), NULL);
-  g_return_val_if_fail (gimp_is_canonical_identifier (procedure_name), NULL);
-
-  va_start (args, first_type);
-
-  return_values = gimp_pdb_run_procedure_valist (pdb, procedure_name,
-                                                 first_type, args);
-
-  va_end (args);
-
-  return return_values;
-}
-
-/**
- * gimp_pdb_run_procedure_valist: (skip)
- * @pdb:            the #GimpPDB object.
- * @procedure_name: the procedure registered name.
- * @first_type:     the #GType of the first argument, or #G_TYPE_NONE.
- * @args:           the call arguments.
- *
- * Runs the procedure named @procedure_name with @args given in the
- * order as passed to [method@PDB.run_procedure].
- *
- * Returns: (transfer full): the return values for the procedure call.
- *
- * Since: 3.0
- */
-GimpValueArray *
-gimp_pdb_run_procedure_valist (GimpPDB     *pdb,
-                               const gchar *procedure_name,
-                               GType        first_type,
-                               va_list      args)
-{
-  GimpValueArray *arguments;
-  GimpValueArray *return_values;
-  gchar          *error_msg = NULL;
-
-  g_return_val_if_fail (GIMP_IS_PDB (pdb), NULL);
-  g_return_val_if_fail (gimp_is_canonical_identifier (procedure_name), NULL);
-
-  arguments = gimp_value_array_new_from_types_valist (&error_msg,
-                                                      first_type,
-                                                      args);
-
-  if (! arguments)
-    {
-      GError *error = g_error_new_literal (GIMP_PDB_ERROR,
-                                           GIMP_PDB_ERROR_INTERNAL_ERROR,
-                                           error_msg);
-      g_printerr ("%s: %s", G_STRFUNC, error_msg);
-      g_free (error_msg);
-
-      return gimp_procedure_new_return_values (NULL,
-                                               GIMP_PDB_CALLING_ERROR,
-                                               error);
-    }
-
-  return_values = gimp_pdb_run_procedure_array (pdb, procedure_name,
-                                                arguments);
-  gimp_value_array_unref (arguments);
-
-  return return_values;
-}
-
-/**
- * gimp_pdb_run_procedure_argv: (rename-to gimp_pdb_run_procedure)
- * @pdb:                                              the #GimpPDB object.
- * @procedure_name:                                   the registered name to call.
- * @arguments: (array length=n_arguments) (nullable): the call arguments or %NULL.
- * @n_arguments:                                      the number of arguments.
- *
- * Runs the procedure named @procedure_name with @arguments.
- *
- * Returns: (transfer full): the return values for the procedure call.
- *
- * Since: 3.0
- */
-GimpValueArray *
-gimp_pdb_run_procedure_argv (GimpPDB      *pdb,
-                             const gchar  *procedure_name,
-                             const GValue *arguments,
-                             gint          n_arguments)
-{
-  GimpValueArray *args;
-  GimpValueArray *return_values;
-
-  g_return_val_if_fail (GIMP_IS_PDB (pdb), NULL);
-  g_return_val_if_fail (gimp_is_canonical_identifier (procedure_name), NULL);
-  /* Not require arguments != NULL.
-   * gimp_value_array_new_from_values(NULL, 0) will return empty GValueArray.
-   */
-
-  args = gimp_value_array_new_from_values (arguments, n_arguments);
-  return_values = gimp_pdb_run_procedure_array (pdb, procedure_name, args);
-  gimp_value_array_unref (args);
-
-  return return_values;
-}
-
-/**
- * gimp_pdb_run_procedure_array:
- * @pdb:            the #GimpPDB object.
- * @procedure_name: the procedure registered name.
- * @arguments:      the call arguments.
- *
- * Runs the procedure named @procedure_name with @arguments.
- *
- * Returns: (transfer full): the return values for the procedure call.
- *
- * Since: 3.0
- */
-GimpValueArray *
-gimp_pdb_run_procedure_array (GimpPDB              *pdb,
-                              const gchar          *procedure_name,
-                              const GimpValueArray *arguments)
-{
-  GPProcRun        proc_run;
-  GPProcReturn    *proc_return;
-  GimpWireMessage  msg;
-  GimpValueArray  *return_values;
-
-  g_return_val_if_fail (GIMP_IS_PDB (pdb), NULL);
-  g_return_val_if_fail (gimp_is_canonical_identifier (procedure_name), NULL);
-  g_return_val_if_fail (arguments != NULL, NULL);
-
-  proc_run.name     = (gchar *) procedure_name;
-  proc_run.n_params = gimp_value_array_length (arguments);
-  proc_run.params   = _gimp_value_array_to_gp_params (arguments, FALSE);
-
-  if (! gp_proc_run_write (_gimp_plug_in_get_write_channel (pdb->priv->plug_in),
-                           &proc_run, pdb->priv->plug_in))
-    gimp_quit ();
-
-  _gimp_gp_params_free (proc_run.params, proc_run.n_params, FALSE);
-
-  _gimp_plug_in_read_expect_msg (pdb->priv->plug_in, &msg, GP_PROC_RETURN);
-
-  proc_return = msg.data;
-
-  return_values = _gimp_gp_params_to_value_array (NULL,
-                                                  NULL, 0,
-                                                  proc_return->params,
-                                                  proc_return->n_params,
-                                                  TRUE);
-
-  gimp_wire_destroy (&msg);
-
-  gimp_pdb_set_error (pdb, return_values);
-
-  return return_values;
-}
-
-/**
- * gimp_pdb_run_procedure_config:
- * @pdb:            the #GimpPDB object.
- * @procedure_name: the registered name to call.
- * @config:         a config object obtained with gimp_procedure_create_config().
- *
- * Runs the procedure named @procedure_name with @config.
- *
- * Returns: (transfer full): the return values for the procedure call.
- *
- * Since: 3.0
- */
-GimpValueArray *
-gimp_pdb_run_procedure_config (GimpPDB             *pdb,
-                               const gchar         *procedure_name,
-                               GimpProcedureConfig *config)
-{
-  GimpProcedure  *procedure;
-  GimpValueArray *args;
-  GimpValueArray *return_values;
-
-  g_return_val_if_fail (GIMP_IS_PDB (pdb), NULL);
-  g_return_val_if_fail (gimp_is_canonical_identifier (procedure_name), NULL);
-  g_return_val_if_fail (GIMP_IS_PROCEDURE_CONFIG (config), NULL);
-
-  procedure = gimp_pdb_lookup_procedure (pdb, procedure_name);
-
-  g_return_val_if_fail (gimp_procedure_config_get_procedure (config) == procedure,
-                        NULL);
-
-  args = gimp_procedure_new_arguments (procedure);
-
-  gimp_procedure_config_get_values (config, args);
-  return_values = gimp_pdb_run_procedure_array (pdb, procedure_name, args);
-
-  gimp_value_array_unref (args);
-
-  return return_values;
-}
-
 /**
  * gimp_pdb_temp_procedure_name:
  * @pdb: the #GimpPDB object.
@@ -516,8 +301,6 @@ _gimp_pdb_error_quark (void)
 }
 
 
-/*  Temporary API, to go away before 3.0  */
-
 /**
  * gimp_pdb_get_last_error:
  * @pdb: a #GimpPDB.
@@ -542,10 +325,10 @@ gimp_pdb_get_last_error (GimpPDB *pdb)
 {
   g_return_val_if_fail (GIMP_IS_PDB (pdb), NULL);
 
-  if (pdb->priv->error_message && strlen (pdb->priv->error_message))
-    return pdb->priv->error_message;
+  if (pdb->error_message && strlen (pdb->error_message))
+    return pdb->error_message;
 
-  switch (pdb->priv->error_status)
+  switch (pdb->error_status)
     {
     case GIMP_PDB_SUCCESS:
       /*  procedure executed successfully  */
@@ -583,7 +366,7 @@ gimp_pdb_get_last_status (GimpPDB *pdb)
 {
   g_return_val_if_fail (GIMP_IS_PDB (pdb), GIMP_PDB_SUCCESS);
 
-  return pdb->priv->error_status;
+  return pdb->error_status;
 }
 
 /*  Cruft API  */
@@ -599,43 +382,19 @@ gimp_pdb_get_last_status (GimpPDB *pdb)
  * the specified identifier. The data is copied into the given memory
  * location.
  *
+ * WARNING: this function is exported in the library so that it can be used by
+ * libgimpwidgets. Nevertheless it is considered internal, and is not declared
+ * in any public header on purpose. It should not be considered part of the API
+ * and therefore should not be used in plug-ins. It may disappear at any time.
+ *
  * Returns: TRUE on success, FALSE if no data has been associated with
  * the identifier
  */
 gboolean
-gimp_pdb_get_data (const gchar *identifier,
-                   gpointer     data)
+gimp_pdb_get_data (const gchar  *identifier,
+                   GBytes      **data)
 {
-  GBytes   *hack = NULL;
-  gboolean  success;
-
-  success = _gimp_pdb_get_data (identifier, &hack);
-
-  if (hack)
-    {
-      memcpy (data, g_bytes_get_data (hack, NULL), g_bytes_get_size (hack));
-      g_free (hack);
-    }
-
-  return success;
-}
-
-/**
- * gimp_pdb_get_data_size:
- * @identifier: The identifier associated with data.
- *
- * Returns size of data associated with the specified identifier.
- *
- * This procedure returns the size of any data which may have been
- * associated with the specified identifier. If no data has been
- * associated with the identifier, an error is returned.
- *
- * Returns: The number of bytes in the data.
- **/
-gint
-gimp_pdb_get_data_size (const gchar *identifier)
-{
-  return _gimp_pdb_get_data_size (identifier);
+  return _gimp_pdb_get_data (identifier, data);
 }
 
 /**
@@ -648,23 +407,74 @@ gimp_pdb_get_data_size (const gchar *identifier)
  *
  * This procedure associates the supplied data with the provided
  * identifier. The data may be subsequently retrieved by a call to
- * 'procedural-db-get-data'.
+ * 'procedural-db-get-data'. This storage is global within the session, even
+ * shareable between plug-ins, though it won't survive a restart of GIMP.
  *
+ * WARNING: this function is exported in the library so that it can be used by
+ * libgimpwidgets. Nevertheless it is considered internal, and is not declared
+ * in any public header on purpose. It should not be considered part of the API
+ * and therefore should not be used in plug-ins. It may disappear at any time.
+
  * Returns: TRUE on success.
  */
 gboolean
-gimp_pdb_set_data (const gchar   *identifier,
-                   gconstpointer  data,
-                   guint32        data_len)
+gimp_pdb_set_data (const gchar *identifier,
+                   GBytes      *data)
 {
-  GBytes   *bytes;
-  gboolean  ret;
+  return _gimp_pdb_set_data (identifier, data);
+}
 
-  bytes = g_bytes_new_static (data, data_len);
-  ret = _gimp_pdb_set_data (identifier, bytes);
-  g_bytes_unref (bytes);
+/**
+ * _gimp_pdb_run_procedure_array:
+ * @pdb:            the #GimpPDB object.
+ * @procedure_name: the procedure registered name.
+ * @arguments:      the call arguments.
+ *
+ * Runs the procedure named @procedure_name with @arguments.
+ *
+ * Returns: (transfer full): the return values for the procedure call.
+ *
+ * Since: 3.0
+ */
+GimpValueArray *
+_gimp_pdb_run_procedure_array (GimpPDB              *pdb,
+                               const gchar          *procedure_name,
+                               const GimpValueArray *arguments)
+{
+  GPProcRun        proc_run;
+  GPProcReturn    *proc_return;
+  GimpWireMessage  msg;
+  GimpValueArray  *return_values;
 
-  return ret;
+  g_return_val_if_fail (GIMP_IS_PDB (pdb), NULL);
+  g_return_val_if_fail (gimp_is_canonical_identifier (procedure_name), NULL);
+  g_return_val_if_fail (arguments != NULL, NULL);
+
+  proc_run.name     = (gchar *) procedure_name;
+  proc_run.n_params = gimp_value_array_length (arguments);
+  proc_run.params   = _gimp_value_array_to_gp_params (arguments, FALSE);
+
+  if (! gp_proc_run_write (_gimp_plug_in_get_write_channel (pdb->plug_in),
+                           &proc_run, pdb->plug_in))
+    gimp_quit ();
+
+  _gimp_gp_params_free (proc_run.params, proc_run.n_params, FALSE);
+
+  _gimp_plug_in_read_expect_msg (pdb->plug_in, &msg, GP_PROC_RETURN);
+
+  proc_return = msg.data;
+
+  return_values = _gimp_gp_params_to_value_array (NULL,
+                                                  NULL, 0,
+                                                  proc_return->params,
+                                                  proc_return->n_params,
+                                                  TRUE);
+
+  gimp_wire_destroy (&msg);
+
+  gimp_pdb_set_error (pdb, return_values);
+
+  return return_values;
 }
 
 
@@ -674,14 +484,14 @@ static void
 gimp_pdb_set_error (GimpPDB        *pdb,
                     GimpValueArray *return_values)
 {
-  g_clear_pointer (&pdb->priv->error_message, g_free);
-  pdb->priv->error_status = GIMP_PDB_SUCCESS;
+  g_clear_pointer (&pdb->error_message, g_free);
+  pdb->error_status = GIMP_PDB_SUCCESS;
 
   if (gimp_value_array_length (return_values) > 0)
     {
-      pdb->priv->error_status = GIMP_VALUES_GET_ENUM (return_values, 0);
+      pdb->error_status = GIMP_VALUES_GET_ENUM (return_values, 0);
 
-      switch (pdb->priv->error_status)
+      switch (pdb->error_status)
         {
         case GIMP_PDB_SUCCESS:
         case GIMP_PDB_PASS_THROUGH:
@@ -695,7 +505,7 @@ gimp_pdb_set_error (GimpPDB        *pdb,
               GValue *value = gimp_value_array_index (return_values, 1);
 
               if (G_VALUE_HOLDS_STRING (value))
-                pdb->priv->error_message = g_value_dup_string (value);
+                pdb->error_message = g_value_dup_string (value);
             }
           break;
         }
