@@ -44,7 +44,7 @@ struct _CropClass
 };
 
 #define CROP_TYPE  (crop_get_type ())
-#define CROP (obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), CROP_TYPE, Crop))
+#define CROP(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), CROP_TYPE, Crop))
 
 GType                   crop_get_type         (void) G_GNUC_CONST;
 
@@ -55,9 +55,8 @@ static GimpProcedure  * crop_create_procedure (GimpPlugIn           *plug_in,
 static GimpValueArray * crop_run              (GimpProcedure        *procedure,
                                                GimpRunMode           run_mode,
                                                GimpImage            *image,
-                                               gint                  n_drawables,
                                                GimpDrawable        **drawables,
-                                               const GimpValueArray *args,
+                                               GimpProcedureConfig  *config,
                                                gpointer              run_data);
 
 static inline gboolean  colors_equal          (const gfloat         *col1,
@@ -112,7 +111,7 @@ crop_create_procedure (GimpPlugIn  *plug_in,
                                            GIMP_PROCEDURE_SENSITIVE_DRAWABLE);
 
       gimp_procedure_set_menu_label (procedure, _("_Zealous Crop"));
-      gimp_procedure_add_menu_path (procedure, "<Image>/Image");
+      gimp_procedure_add_menu_path (procedure, "<Image>/Image/[Crop]");
 
       gimp_procedure_set_documentation (procedure,
                                         _("Autocrop unused space from "
@@ -132,9 +131,8 @@ static GimpValueArray *
 crop_run (GimpProcedure        *procedure,
           GimpRunMode           run_mode,
           GimpImage            *image,
-          gint                  n_drawables,
           GimpDrawable        **drawables,
-          const GimpValueArray *args,
+          GimpProcedureConfig  *config,
           gpointer              run_data)
 {
   GimpDrawable *drawable;
@@ -143,7 +141,7 @@ crop_run (GimpProcedure        *procedure,
 
   gimp_progress_init (_("Zealous cropping"));
 
-  if (n_drawables != 1)
+  if (gimp_core_object_array_get_length ((GObject **) drawables) != 1)
     {
       GError *error = NULL;
 
