@@ -34,7 +34,7 @@
 
 #define LOAD_PROC      "file-pcx-load"
 #define LOAD_PROC_DCX  "file-dcx-load"
-#define SAVE_PROC      "file-pcx-save"
+#define EXPORT_PROC    "file-pcx-export"
 #define PLUG_IN_BINARY "file-pcx"
 #define PLUG_IN_ROLE   "gimp-file-pcx"
 
@@ -54,109 +54,113 @@ struct _PcxClass
 
 
 #define PCX_TYPE  (pcx_get_type ())
-#define PCX (obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), PCX_TYPE, Pcx))
+#define PCX(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), PCX_TYPE, Pcx))
 
 GType                   pcx_get_type         (void) G_GNUC_CONST;
 
-static GList          * pcx_query_procedures (GimpPlugIn           *plug_in);
-static GimpProcedure  * pcx_create_procedure (GimpPlugIn           *plug_in,
-                                              const gchar          *name);
+static GList          * pcx_query_procedures (GimpPlugIn            *plug_in);
+static GimpProcedure  * pcx_create_procedure (GimpPlugIn            *plug_in,
+                                              const gchar           *name);
 
-static GimpValueArray * pcx_load             (GimpProcedure        *procedure,
-                                              GimpRunMode           run_mode,
-                                              GFile                *file,
-                                              const GimpValueArray *args,
-                                              gpointer              run_data);
-static GimpValueArray * dcx_load             (GimpProcedure        *procedure,
-                                              GimpRunMode           run_mode,
-                                              GFile                *file,
-                                              const GimpValueArray *args,
-                                              gpointer              run_data);
-static GimpValueArray * pcx_save             (GimpProcedure        *procedure,
-                                              GimpRunMode           run_mode,
-                                              GimpImage            *image,
-                                              gint                  n_drawables,
-                                              GimpDrawable        **drawables,
-                                              GFile                *file,
-                                              const GimpValueArray *args,
-                                              gpointer              run_data);
+static GimpValueArray * pcx_load             (GimpProcedure         *procedure,
+                                              GimpRunMode            run_mode,
+                                              GFile                 *file,
+                                              GimpMetadata          *metadata,
+                                              GimpMetadataLoadFlags *flags,
+                                              GimpProcedureConfig   *config,
+                                              gpointer               run_data);
+static GimpValueArray * dcx_load             (GimpProcedure         *procedure,
+                                              GimpRunMode            run_mode,
+                                              GFile                 *file,
+                                              GimpMetadata          *metadata,
+                                              GimpMetadataLoadFlags *flags,
+                                              GimpProcedureConfig   *config,
+                                              gpointer               run_data);
+static GimpValueArray * pcx_export           (GimpProcedure         *procedure,
+                                              GimpRunMode            run_mode,
+                                              GimpImage             *image,
+                                              GFile                 *file,
+                                              GimpExportOptions     *options,
+                                              GimpMetadata          *metadata,
+                                              GimpProcedureConfig   *config,
+                                              gpointer               run_data);
 
-static GimpImage      * load_single          (GimpProcedure        *procedure,
-                                              GFile                *file,
-                                              GObject              *config,
-                                              gint                  run_mode,
-                                              GError              **error);
-static GimpImage      * load_multi           (GimpProcedure        *procedure,
-                                              GFile                *file,
-                                              GObject              *config,
-                                              gint                  run_mode,
-                                              GError              **error);
+static GimpImage      * load_single          (GimpProcedure         *procedure,
+                                              GFile                 *file,
+                                              GObject               *config,
+                                              gint                   run_mode,
+                                              GError               **error);
+static GimpImage      * load_multi           (GimpProcedure         *procedure,
+                                              GFile                 *file,
+                                              GObject               *config,
+                                              gint                   run_mode,
+                                              GError               **error);
 
-static GimpImage      * load_image           (GimpProcedure        *procedure,
-                                              FILE                 *fd,
-                                              const gchar          *filename,
-                                              GObject              *config,
-                                              gint                  run_mode,
-                                              gint                  image_num,
-                                              GError              **error);
-static gboolean         pcx_load_dialog      (GimpProcedure        *procedure,
-                                              GObject              *config);
+static GimpImage      * load_image           (GimpProcedure         *procedure,
+                                              FILE                  *fd,
+                                              const gchar           *filename,
+                                              GObject               *config,
+                                              gint                   run_mode,
+                                              gint                   image_num,
+                                              GError               **error);
+static gboolean         pcx_load_dialog      (GimpProcedure         *procedure,
+                                              GObject               *config);
 
-static void             load_1               (FILE                 *fp,
-                                              gint                  width,
-                                              gint                  height,
-                                              guchar               *buf,
-                                              guint16               bytes);
-static void             load_4               (FILE                 *fp,
-                                              gint                  width,
-                                              gint                  height,
-                                              guchar               *buf,
-                                              guint16               bytes);
-static void             load_sub_8           (FILE                 *fp,
-                                              gint                  width,
-                                              gint                  height,
-                                              gint                  bpp,
-                                              gint                  plane,
-                                              guchar               *buf,
-                                              guint16               bytes);
-static void             load_8               (FILE                 *fp,
-                                              gint                  width,
-                                              gint                  height,
-                                              guchar               *buf,
-                                              guint16               bytes);
-static void             load_24              (FILE                 *fp,
-                                              gint                  width,
-                                              gint                  height,
-                                              guchar               *buf,
-                                              guint16               bytes,
-                                              guint8                planes);
-static void             readline             (FILE                 *fp,
-                                              guchar               *buf,
-                                              gint                  bytes);
+static void             load_1               (FILE                  *fp,
+                                              gint                   width,
+                                              gint                   height,
+                                              guchar                *buf,
+                                              guint16                bytes);
+static void             load_4               (FILE                  *fp,
+                                              gint                   width,
+                                              gint                   height,
+                                              guchar                *buf,
+                                              guint16                bytes);
+static void             load_sub_8           (FILE                  *fp,
+                                              gint                   width,
+                                              gint                   height,
+                                              gint                   bpp,
+                                              gint                   plane,
+                                              guchar                *buf,
+                                              guint16                bytes);
+static void             load_8               (FILE                  *fp,
+                                              gint                   width,
+                                              gint                   height,
+                                              guchar                *buf,
+                                              guint16                bytes);
+static void             load_24              (FILE                  *fp,
+                                              gint                   width,
+                                              gint                   height,
+                                              guchar                *buf,
+                                              guint16                bytes,
+                                              guint8                 planes);
+static void             readline             (FILE                  *fp,
+                                              guchar                *buf,
+                                              gint                   bytes);
 
-static gboolean         save_image           (GFile                *file,
-                                              GimpImage            *image,
-                                              GimpDrawable         *drawable,
-                                              GError              **error);
-static void             save_less_than_8     (FILE                 *fp,
-                                              gint                  width,
-                                              gint                  height,
-                                              const gint            bpp,
-                                              const guchar         *buf,
-                                              gboolean              padding);
-static void             save_8               (FILE                 *fp,
-                                              gint                  width,
-                                              gint                  height,
-                                              const guchar         *buf,
-                                              gboolean              padding);
-static void             save_24              (FILE                 *fp,
-                                              gint                  width,
-                                              gint                  height,
-                                              const guchar         *buf,
-                                              gboolean              padding);
-static void             writeline            (FILE                 *fp,
-                                              const guchar         *buf,
-                                              gint                  bytes);
+static gboolean         export_image         (GFile                 *file,
+                                              GimpImage             *image,
+                                              GimpDrawable          *drawable,
+                                              GError               **error);
+static void             save_less_than_8     (FILE                  *fp,
+                                              gint                   width,
+                                              gint                   height,
+                                              const gint             bpp,
+                                              const guchar          *buf,
+                                              gboolean               padding);
+static void             save_8               (FILE                  *fp,
+                                              gint                   width,
+                                              gint                   height,
+                                              const guchar          *buf,
+                                              gboolean               padding);
+static void             save_24              (FILE                  *fp,
+                                              gint                   width,
+                                              gint                   height,
+                                              const guchar          *buf,
+                                              gboolean               padding);
+static void             writeline            (FILE                  *fp,
+                                              const guchar          *buf,
+                                              gint                   bytes);
 
 
 G_DEFINE_TYPE (Pcx, pcx, GIMP_TYPE_PLUG_IN)
@@ -187,7 +191,7 @@ pcx_query_procedures (GimpPlugIn *plug_in)
 
   list = g_list_append (list, g_strdup (LOAD_PROC));
   list = g_list_append (list, g_strdup (LOAD_PROC_DCX));
-  list = g_list_append (list, g_strdup (SAVE_PROC));
+  list = g_list_append (list, g_strdup (EXPORT_PROC));
 
   return list;
 }
@@ -215,12 +219,15 @@ pcx_create_procedure (GimpPlugIn  *plug_in,
                                       "Nick Lamb <njl195@zepler.org.uk>",
                                       "January 1997");
 
-      GIMP_PROC_ARG_INT (procedure, "override-palette",
-                         _("Palette Options"),
-                         _("Use built-in palette (0) or override with "
-                           "black/white (1)"),
-                         0, 1, 0,
-                         G_PARAM_READWRITE);
+      gimp_procedure_add_choice_argument (procedure, "override-palette",
+                                          _("_Palette Options"),
+                                          _("Whether to use the built-in palette or "
+                                            "a black and white palette for 1 bit images."),
+                                          gimp_choice_new_with_values ("use-built-in-palette", 0, _("Use PCX image's built-in palette"), NULL,
+                                                                       "use-bw-palette",       1, _("Use black and white palette"),      NULL,
+                                                                       NULL),
+                                          "use-built-in-palette",
+                                          G_PARAM_READWRITE);
 
       gimp_file_procedure_set_mime_types (GIMP_FILE_PROCEDURE (procedure),
                                           "image/x-pcx");
@@ -246,12 +253,15 @@ pcx_create_procedure (GimpPlugIn  *plug_in,
                                       "Alex S.",
                                       "2023");
 
-      GIMP_PROC_ARG_INT (procedure, "override-palette",
-                         _("Palette Options"),
-                         _("Use built-in palette (0) or override with "
-                           "black/white (1)"),
-                         0, 1, 0,
-                         G_PARAM_READWRITE);
+      gimp_procedure_add_choice_argument (procedure, "override-palette",
+                                          _("_Palette Options"),
+                                          _("Whether to use the built-in palette or "
+                                            "a black and white palette for 1 bit images."),
+                                          gimp_choice_new_with_values ("use-built-in-palette", 0, _("Use PCX image's built-in palette"), NULL,
+                                                                       "use-bw-palette",       1, _("Use black and white palette"),      NULL,
+                                                                       NULL),
+                                          "use-built-in-palette",
+                                          G_PARAM_READWRITE);
 
       gimp_file_procedure_set_mime_types (GIMP_FILE_PROCEDURE (procedure),
                                           "image/x-dcx");
@@ -260,19 +270,19 @@ pcx_create_procedure (GimpPlugIn  *plug_in,
       gimp_file_procedure_set_magics (GIMP_FILE_PROCEDURE (procedure),
                                       "0,string,\xB1\x68\xDE\x3A");
     }
-  else if (! strcmp (name, SAVE_PROC))
+  else if (! strcmp (name, EXPORT_PROC))
     {
-      procedure = gimp_save_procedure_new (plug_in, name,
-                                           GIMP_PDB_PROC_TYPE_PLUGIN,
-                                           pcx_save, NULL, NULL);
+      procedure = gimp_export_procedure_new (plug_in, name,
+                                             GIMP_PDB_PROC_TYPE_PLUGIN,
+                                             FALSE, pcx_export, NULL, NULL);
 
       gimp_procedure_set_image_types (procedure, "INDEXED, RGB, GRAY");
 
       gimp_procedure_set_menu_label (procedure, _("ZSoft PCX image"));
 
       gimp_procedure_set_documentation (procedure,
-                                        "Exports files in ZSoft PCX file format",
-                                        "FIXME: write help for pcx_save",
+                                        _("Exports files in ZSoft PCX file format"),
+                                        "FIXME: write help for pcx_export",
                                         name);
       gimp_procedure_set_attribution (procedure,
                                       "Francisco Bustamante & Nick Lamb",
@@ -283,42 +293,38 @@ pcx_create_procedure (GimpPlugIn  *plug_in,
                                           "image/x-pcx");
       gimp_file_procedure_set_extensions (GIMP_FILE_PROCEDURE (procedure),
                                           "pcx,pcc");
+
+      gimp_export_procedure_set_capabilities (GIMP_EXPORT_PROCEDURE (procedure),
+                                              GIMP_EXPORT_CAN_HANDLE_RGB  |
+                                              GIMP_EXPORT_CAN_HANDLE_GRAY |
+                                              GIMP_EXPORT_CAN_HANDLE_INDEXED,
+                                              NULL, NULL, NULL);
     }
 
   return procedure;
 }
 
 static GimpValueArray *
-pcx_load (GimpProcedure        *procedure,
-          GimpRunMode           run_mode,
-          GFile                *file,
-          const GimpValueArray *args,
-          gpointer              run_data)
+pcx_load (GimpProcedure         *procedure,
+          GimpRunMode            run_mode,
+          GFile                 *file,
+          GimpMetadata          *metadata,
+          GimpMetadataLoadFlags *flags,
+          GimpProcedureConfig   *config,
+          gpointer               run_data)
 {
-  GimpProcedureConfig *config;
-  GimpValueArray      *return_vals;
-  GimpImage           *image;
-  GError              *error = NULL;
+  GimpValueArray *return_vals;
+  GimpImage      *image;
+  GError         *error = NULL;
 
   gegl_init (NULL, NULL);
 
-  config = gimp_procedure_create_config (procedure);
-  gimp_procedure_config_begin_run (config, NULL, run_mode, args);
-
   image = load_single (procedure, file, G_OBJECT (config), run_mode, &error);
 
   if (! image)
-    {
-      gimp_procedure_config_end_run (config, GIMP_PDB_CANCEL);
-      g_object_unref (config);
-
-      return gimp_procedure_new_return_values (procedure,
-                                               GIMP_PDB_EXECUTION_ERROR,
-                                               error);
-    }
-
-  gimp_procedure_config_end_run (config, GIMP_PDB_SUCCESS);
-  g_object_unref (config);
+    return gimp_procedure_new_return_values (procedure,
+                                             GIMP_PDB_EXECUTION_ERROR,
+                                             error);
 
   return_vals = gimp_procedure_new_return_values (procedure,
                                                   GIMP_PDB_SUCCESS,
@@ -330,36 +336,26 @@ pcx_load (GimpProcedure        *procedure,
 }
 
 static GimpValueArray *
-dcx_load (GimpProcedure        *procedure,
-          GimpRunMode           run_mode,
-          GFile                *file,
-          const GimpValueArray *args,
-          gpointer              run_data)
+dcx_load (GimpProcedure         *procedure,
+          GimpRunMode            run_mode,
+          GFile                 *file,
+          GimpMetadata          *metadata,
+          GimpMetadataLoadFlags *flags,
+          GimpProcedureConfig   *config,
+          gpointer               run_data)
 {
-  GimpProcedureConfig *config;
-  GimpValueArray      *return_vals;
-  GimpImage           *image;
-  GError              *error = NULL;
+  GimpValueArray *return_vals;
+  GimpImage      *image;
+  GError         *error = NULL;
 
   gegl_init (NULL, NULL);
 
-  config = gimp_procedure_create_config (procedure);
-  gimp_procedure_config_begin_run (config, NULL, run_mode, args);
-
   image = load_multi (procedure, file, G_OBJECT (config), run_mode, &error);
 
   if (! image)
-    {
-      gimp_procedure_config_end_run (config, GIMP_PDB_CANCEL);
-      g_object_unref (config);
-
-      return gimp_procedure_new_return_values (procedure,
-                                               GIMP_PDB_EXECUTION_ERROR,
-                                               error);
-    }
-
-  gimp_procedure_config_end_run (config, GIMP_PDB_SUCCESS);
-  g_object_unref (config);
+    return gimp_procedure_new_return_values (procedure,
+                                             GIMP_PDB_EXECUTION_ERROR,
+                                             error);
 
   return_vals = gimp_procedure_new_return_values (procedure,
                                                   GIMP_PDB_SUCCESS,
@@ -371,65 +367,36 @@ dcx_load (GimpProcedure        *procedure,
 }
 
 static GimpValueArray *
-pcx_save (GimpProcedure        *procedure,
-          GimpRunMode           run_mode,
-          GimpImage            *image,
-          gint                  n_drawables,
-          GimpDrawable        **drawables,
-          GFile                *file,
-          const GimpValueArray *args,
-          gpointer              run_data)
+pcx_export (GimpProcedure        *procedure,
+            GimpRunMode           run_mode,
+            GimpImage            *image,
+            GFile                *file,
+            GimpExportOptions    *options,
+            GimpMetadata         *metadata,
+            GimpProcedureConfig  *config,
+            gpointer              run_data)
 {
   GimpPDBStatusType  status = GIMP_PDB_SUCCESS;
-  GimpExportReturn   export = GIMP_EXPORT_CANCEL;
+  GimpExportReturn   export = GIMP_EXPORT_IGNORE;
+  GList             *drawables;
   GError            *error  = NULL;
 
   gegl_init (NULL, NULL);
 
-  switch (run_mode)
-    {
-    case GIMP_RUN_INTERACTIVE:
-    case GIMP_RUN_WITH_LAST_VALS:
-      gimp_ui_init (PLUG_IN_BINARY);
+  export = gimp_export_options_get_image (options, &image);
+  drawables = gimp_image_list_layers (image);
 
-      export = gimp_export_image (&image, &n_drawables, &drawables, "PCX",
-                                  GIMP_EXPORT_CAN_HANDLE_RGB  |
-                                  GIMP_EXPORT_CAN_HANDLE_GRAY |
-                                  GIMP_EXPORT_CAN_HANDLE_INDEXED);
-
-      if (export == GIMP_EXPORT_CANCEL)
-        return gimp_procedure_new_return_values (procedure,
-                                                 GIMP_PDB_CANCEL,
-                                                 NULL);
-      break;
-
-    default:
-      break;
-    }
-
-  if (n_drawables != 1)
-    {
-      g_set_error (&error, G_FILE_ERROR, 0,
-                   _("PCX format does not support multiple layers."));
-
-      return gimp_procedure_new_return_values (procedure,
-                                               GIMP_PDB_CALLING_ERROR,
-                                               error);
-    }
-
-  if (! save_image (file,
-                    image, drawables[0],
-                    &error))
+  if (! export_image (file,
+                      image, drawables->data,
+                      &error))
     {
       status = GIMP_PDB_EXECUTION_ERROR;
     }
 
   if (export == GIMP_EXPORT_EXPORT)
-    {
-      gimp_image_delete (image);
-      g_free (drawables);
-    }
+    gimp_image_delete (image);
 
+  g_list_free (drawables);
   return gimp_procedure_new_return_values (procedure, status, error);
 }
 
@@ -531,8 +498,8 @@ load_single (GimpProcedure  *procedure,
 
   if (! image)
     {
-      g_set_error (error, G_FILE_ERROR, g_file_error_from_errno (errno),
-                   _("Could not load PCX image"));
+      g_prefix_error (error,
+                      _("Could not load PCX image: "));
       return NULL;
     }
 
@@ -589,8 +556,8 @@ load_multi (GimpProcedure  *procedure,
   if (! image)
     {
       fclose (fd);
-      g_set_error (error, G_FILE_ERROR, g_file_error_from_errno (errno),
-                   _("Could not load DCX image"));
+      g_prefix_error (error,
+                      _("Could not load DCX image: "));
       return NULL;
     }
 
@@ -602,7 +569,6 @@ load_multi (GimpProcedure  *procedure,
     {
       GimpLayer **layers;
       GimpLayer  *new_layer;
-      gint        n_layers;
 
       fread (&offset, 1, 4, fd);
 
@@ -624,7 +590,7 @@ load_multi (GimpProcedure  *procedure,
 
       if (temp_image)
         {
-          layers = gimp_image_get_layers (temp_image, &n_layers);
+          layers = gimp_image_get_layers (temp_image);
           new_layer = gimp_layer_new_from_drawable (GIMP_DRAWABLE (layers[0]),
                                                     image);
           gimp_item_set_name (GIMP_ITEM (new_layer),
@@ -679,7 +645,6 @@ load_image (GimpProcedure  *procedure,
       g_set_error (error, G_FILE_ERROR, G_FILE_ERROR_FAILED,
                    _("Could not read header from '%s'"),
                    filename);
-      fclose (fd);
       return NULL;
     }
 
@@ -690,7 +655,6 @@ load_image (GimpProcedure  *procedure,
       g_set_error (error, G_FILE_ERROR, G_FILE_ERROR_FAILED,
                    _("'%s' is not a PCX file"),
                    filename);
-      fclose (fd);
       return NULL;
     }
 
@@ -704,20 +668,23 @@ load_image (GimpProcedure  *procedure,
 
   if ((width <= 0) || (width > GIMP_MAX_IMAGE_SIZE))
     {
-      g_message (_("Unsupported or invalid image width: %d"), width);
-      fclose (fd);
+      g_set_error (error, GIMP_PLUG_IN_ERROR, 0,
+                   _("Unsupported or invalid image width: %d"),
+                   width);
       return NULL;
     }
   if ((height <= 0) || (height > GIMP_MAX_IMAGE_SIZE))
     {
-      g_message (_("Unsupported or invalid image height: %d"), height);
-      fclose (fd);
+      g_set_error (error, GIMP_PLUG_IN_ERROR, 0,
+                   _("Unsupported or invalid image height: %d"),
+                   height);
       return NULL;
     }
-  if ((bytesperline + 1) < ((width * pcx_header.bpp + 7) / 8))
+  if ((bytesperline + 1) < ((width * pcx_header.bpp + 7) / 8) ||
+      bytesperline == 0)
     {
-      g_message (_("Invalid number of bytes per line in PCX header"));
-      fclose (fd);
+      g_set_error (error, GIMP_PLUG_IN_ERROR, 0,
+                   _("Invalid number of bytes per line in PCX header"));
       return NULL;
     }
   if ((resolution_x < 1) || (resolution_x > GIMP_MAX_RESOLUTION) ||
@@ -731,8 +698,9 @@ load_image (GimpProcedure  *procedure,
   /* Shield against potential buffer overflows in load_*() functions. */
   if (G_MAXSIZE / width / height < 3)
     {
-      g_message (_("Image dimensions too large: width %d x height %d"), width, height);
-      fclose (fd);
+      g_set_error (error, GIMP_PLUG_IN_ERROR, 0,
+                   _("Image dimensions too large: width %d x height %d"),
+                   width, height);
       return NULL;
     }
 
@@ -776,16 +744,13 @@ load_image (GimpProcedure  *procedure,
 
       if (run_mode == GIMP_RUN_INTERACTIVE)
         {
-          g_object_get (config,
-                        "override-palette", &override_palette,
-                        NULL);
+          override_palette = gimp_procedure_config_get_choice_id (GIMP_PROCEDURE_CONFIG (config),
+                                                                  "override-palette");
+
           /* Only show dialogue once for DCX import */
           if (image_num == 0 && pcx_load_dialog (procedure, config))
-            {
-              g_object_get (config,
-                            "override-palette", &override_palette,
-                            NULL);
-            }
+            override_palette = gimp_procedure_config_get_choice_id (GIMP_PROCEDURE_CONFIG (config),
+                                                                    "override-palette");
         }
       /* Monochrome does not mean necessarily B&W. Therefore we still
        * want to check the header palette, even for just 2 colors.
@@ -811,37 +776,49 @@ load_image (GimpProcedure  *procedure,
                                                 255, 255, 255};
           colormap = bw_colormap;
         }
-      gimp_image_set_colormap (image, colormap, 2);
+      gimp_palette_set_colormap (gimp_image_get_palette (image),
+                                 babl_format ("R'G'B' u8"),
+                                 (guint8 *) colormap, 6);
     }
   else if (pcx_header.bpp == 1 && pcx_header.planes == 2)
     {
       dest = g_new (guchar, ((gsize) width) * height);
       load_sub_8 (fd, width, height, 1, 2, dest, bytesperline);
-      gimp_image_set_colormap (image, pcx_header.colormap, 4);
+      gimp_palette_set_colormap (gimp_image_get_palette (image),
+                                 babl_format ("R'G'B' u8"),
+                                 pcx_header.colormap, 4 * 3);
     }
   else if (pcx_header.bpp == 2 && pcx_header.planes == 1)
     {
       dest = g_new (guchar, ((gsize) width) * height);
       load_sub_8 (fd, width, height, 2, 1, dest, bytesperline);
-      gimp_image_set_colormap (image, pcx_header.colormap, 4);
+      gimp_palette_set_colormap (gimp_image_get_palette (image),
+                                 babl_format ("R'G'B' u8"),
+                                 pcx_header.colormap, 4 * 3);
     }
   else if (pcx_header.bpp == 1 && pcx_header.planes == 3)
     {
       dest = g_new (guchar, ((gsize) width) * height);
       load_sub_8 (fd, width, height, 1, 3, dest, bytesperline);
-      gimp_image_set_colormap (image, pcx_header.colormap, 8);
+      gimp_palette_set_colormap (gimp_image_get_palette (image),
+                                 babl_format ("R'G'B' u8"),
+                                 pcx_header.colormap, 8 * 3);
     }
   else if (pcx_header.bpp == 1 && pcx_header.planes == 4)
     {
       dest = g_new (guchar, ((gsize) width) * height);
       load_4 (fd, width, height, dest, bytesperline);
-      gimp_image_set_colormap (image, pcx_header.colormap, 16);
+      gimp_palette_set_colormap (gimp_image_get_palette (image),
+                                 babl_format ("R'G'B' u8"),
+                                 pcx_header.colormap, 16 * 3);
     }
   else if (pcx_header.bpp == 4 && pcx_header.planes == 1)
     {
       dest = g_new (guchar, ((gsize) width) * height);
       load_sub_8 (fd, width, height, 4, 1, dest, bytesperline);
-      gimp_image_set_colormap (image, pcx_header.colormap, 16);
+      gimp_palette_set_colormap (gimp_image_get_palette (image),
+                                 babl_format ("R'G'B' u8"),
+                                 pcx_header.colormap, 16 * 3);
     }
   else if (pcx_header.bpp == 8 && pcx_header.planes == 1)
     {
@@ -849,7 +826,9 @@ load_image (GimpProcedure  *procedure,
       load_8 (fd, width, height, dest, bytesperline);
       fseek (fd, -768L, SEEK_END);
       fread (cmap, 768, 1, fd);
-      gimp_image_set_colormap (image, cmap, 256);
+      gimp_palette_set_colormap (gimp_image_get_palette (image),
+                                 babl_format ("R'G'B' u8"),
+                                 cmap, 256 * 3);
     }
   else if (pcx_header.bpp == 8 && (pcx_header.planes == 3 || pcx_header.planes == 4))
     {
@@ -858,8 +837,9 @@ load_image (GimpProcedure  *procedure,
     }
   else
     {
-      g_message (_("Unusual PCX flavour, giving up"));
-      fclose (fd);
+      g_set_error (error, GIMP_PLUG_IN_ERROR, 0,
+                   _("Unusual PCX flavour, giving up"));
+      g_object_unref (buffer);
       return NULL;
     }
 
@@ -877,7 +857,6 @@ pcx_load_dialog (GimpProcedure *procedure,
                  GObject       *config)
 {
   GtkWidget    *dialog;
-  GtkListStore *store;
   gboolean      run;
 
   gimp_ui_init (PLUG_IN_BINARY);
@@ -886,18 +865,8 @@ pcx_load_dialog (GimpProcedure *procedure,
                                       GIMP_PROCEDURE_CONFIG (config),
                                       _("Import from PCX"));
 
-  gimp_dialog_set_alternative_button_order (GTK_DIALOG (dialog),
-                                            GTK_RESPONSE_OK,
-                                            GTK_RESPONSE_CANCEL,
-                                            -1);
-
-  gimp_window_set_transient (GTK_WINDOW (dialog));
-
-  store = gimp_int_store_new (_("Use PCX image's built-in palette"), 0,
-                              _("Use black and white palette"),      1,
-                              NULL);
-  gimp_procedure_dialog_get_int_radio (GIMP_PROCEDURE_DIALOG (dialog),
-                                       "override-palette", GIMP_INT_STORE (store));
+  gimp_procedure_dialog_get_widget (GIMP_PROCEDURE_DIALOG (dialog),
+                                    "override-palette", GIMP_TYPE_INT_RADIO_FRAME);
 
   gimp_procedure_dialog_fill (GIMP_PROCEDURE_DIALOG (dialog),
                               NULL);
@@ -1085,10 +1054,10 @@ readline (FILE   *fp,
 }
 
 static gboolean
-save_image (GFile         *file,
-            GimpImage     *image,
-            GimpDrawable  *drawable,
-            GError       **error)
+export_image (GFile         *file,
+              GimpImage     *image,
+              GimpDrawable  *drawable,
+              GError       **error)
 {
   FILE          *fp;
   GeglBuffer    *buffer;
@@ -1121,7 +1090,8 @@ save_image (GFile         *file,
   switch (drawable_type)
     {
     case GIMP_INDEXED_IMAGE:
-      cmap = gimp_image_get_colormap (image, NULL, &colors);
+      cmap = gimp_palette_get_colormap (gimp_image_get_palette (image),
+                                        babl_format ("R'G'B' u8"), &colors, NULL);
 
       if (colors > 16)
         {
