@@ -26,7 +26,7 @@
  */
 
 #define LOAD_PROC      "file-psp-load"
-#define SAVE_PROC      "file-psp-save"
+#define EXPORT_PROC    "file-psp-export"
 #define PLUG_IN_BINARY "file-psp"
 #define PLUG_IN_ROLE   "gimp-file-psp"
 
@@ -460,7 +460,7 @@ typedef enum {
   PSP_XDATA_GRID,               /* Image grid information (since PSP7) */
   PSP_XDATA_GUIDE,              /* Image guide information (since PSP7) */
   PSP_XDATA_EXIF,               /* Image Exif information (since PSP8) */
-  PSP_XDATA_IPTC,               /* Image IPTC information (since PSP10) */
+  PSP_XDATA_IPTC,               /* Image IPTC information (since PSP8) */
 } PSPExtendedDataID;
 
 /* Creator field types.
@@ -576,39 +576,41 @@ struct _PspClass
 
 
 #define PSP_TYPE  (psp_get_type ())
-#define PSP (obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), PSP_TYPE, Psp))
+#define PSP(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), PSP_TYPE, Psp))
 
 GType                   psp_get_type         (void) G_GNUC_CONST;
 
-static GList          * psp_query_procedures (GimpPlugIn           *plug_in);
-static GimpProcedure  * psp_create_procedure (GimpPlugIn           *plug_in,
-                                              const gchar          *name);
+static GList          * psp_query_procedures (GimpPlugIn            *plug_in);
+static GimpProcedure  * psp_create_procedure (GimpPlugIn            *plug_in,
+                                              const gchar           *name);
 
-static GimpValueArray * psp_load             (GimpProcedure        *procedure,
-                                              GimpRunMode           run_mode,
-                                              GFile                *file,
-                                              const GimpValueArray *args,
-                                              gpointer              run_data);
-static GimpValueArray * psp_save             (GimpProcedure        *procedure,
-                                              GimpRunMode           run_mode,
-                                              GimpImage            *image,
-                                              gint                  n_drawables,
-                                              GimpDrawable        **drawables,
-                                              GFile                *file,
-                                              const GimpValueArray *args,
-                                              gpointer              run_data);
+static GimpValueArray * psp_load             (GimpProcedure         *procedure,
+                                              GimpRunMode            run_mode,
+                                              GFile                 *file,
+                                              GimpMetadata          *metadata,
+                                              GimpMetadataLoadFlags *flags,
+                                              GimpProcedureConfig   *config,
+                                              gpointer               run_data);
+static GimpValueArray * psp_export           (GimpProcedure         *procedure,
+                                              GimpRunMode            run_mode,
+                                              GimpImage             *image,
+                                              GFile                 *file,
+                                              GimpExportOptions     *options,
+                                              GimpMetadata          *metadata,
+                                              GimpProcedureConfig   *config,
+                                              gpointer               run_data);
 
-static GimpImage      * load_image           (GFile                *file,
-                                              GError              **error);
-static gboolean         save_image           (GFile                *file,
-                                              GimpImage            *image,
-                                              gint                  n_drawables,
-                                              GimpDrawable        **drawables,
-                                              GObject              *config,
-                                              GError              **error);
-static gboolean         save_dialog          (GimpProcedure        *procedure,
-                                              GObject              *config,
-                                              GimpImage            *image);
+static GimpImage      * load_image           (GFile                 *file,
+                                              GError               **error);
+static gboolean         export_image         (GFile                 *file,
+                                              GimpImage             *image,
+                                              gint                   n_drawables,
+                                              GList                 *drawables,
+                                              GObject               *config,
+                                              GError               **error);
+static gboolean         save_dialog          (GimpProcedure         *procedure,
+                                              GObject               *config,
+                                              GimpImage             *image);
 
 
 G_DEFINE_TYPE (Psp, psp, GIMP_TYPE_PLUG_IN)
@@ -644,7 +646,7 @@ psp_query_procedures (GimpPlugIn *plug_in)
   list = g_list_append (list, g_strdup (LOAD_PROC));
 #if 0
   /* commented out until exporting is implemented */
-  list = g_list_append (list, g_strdup (SAVE_PROC));
+  list = g_list_append (list, g_strdup (EXPORT_PROC));
 #endif
   return list;
 }
@@ -683,11 +685,11 @@ psp_create_procedure (GimpPlugIn  *plug_in,
       gimp_file_procedure_set_magics (GIMP_FILE_PROCEDURE (procedure),
                                       "0,string,Paint\\040Shop\\040Pro\\040Image\\040File\n\032");
     }
-  else if (! strcmp (name, SAVE_PROC))
+  else if (! strcmp (name, EXPORT_PROC))
     {
-      procedure = gimp_save_procedure_new (plug_in, name,
-                                           GIMP_PDB_PROC_TYPE_PLUGIN,
-                                           psp_save, NULL, NULL);
+      procedure = gimp_export_procedure_new (plug_in, name,
+                                             GIMP_PDB_PROC_TYPE_PLUGIN,
+                                             FALSE, psp_export, NULL, NULL);
 
       gimp_procedure_set_image_types (procedure, "*");
 
@@ -712,23 +714,36 @@ psp_create_procedure (GimpPlugIn  *plug_in,
       gimp_file_procedure_set_extensions (GIMP_FILE_PROCEDURE (procedure),
                                           "psp,tub");
 
-      GIMP_PROC_ARG_INT (procedure, "compression",
-                         "_Data Compression",
-                         "Specify 0 for no compression, "
-                         "1 for RLE, and 2 for LZ77",
-                         0, 2, PSP_COMP_LZ77,
-                         G_PARAM_READWRITE);
+      gimp_export_procedure_set_capabilities (GIMP_EXPORT_PROCEDURE (procedure),
+                                              GIMP_EXPORT_CAN_HANDLE_RGB     |
+                                              GIMP_EXPORT_CAN_HANDLE_GRAY    |
+                                              GIMP_EXPORT_CAN_HANDLE_INDEXED |
+                                              GIMP_EXPORT_CAN_HANDLE_ALPHA   |
+                                              GIMP_EXPORT_CAN_HANDLE_LAYERS,
+                                              NULL, NULL, NULL);
+
+      gimp_procedure_add_choice_argument (procedure, "compression",
+                                          _("_Data Compression"),
+                                          _("Type of compression"),
+                                          gimp_choice_new_with_values ("none", PSP_COMP_NONE, _("none"), NULL,
+                                                                       "rle",  PSP_COMP_RLE,  _("rle"),  NULL,
+                                                                       "lz77", PSP_COMP_LZ77, _("lz77"), NULL,
+                                                                       NULL),
+                                          "lz77",
+                                          G_PARAM_READWRITE);
     }
 
   return procedure;
 }
 
 static GimpValueArray *
-psp_load (GimpProcedure        *procedure,
-          GimpRunMode           run_mode,
-          GFile                *file,
-          const GimpValueArray *args,
-          gpointer              run_data)
+psp_load (GimpProcedure         *procedure,
+          GimpRunMode            run_mode,
+          GFile                 *file,
+          GimpMetadata          *metadata,
+          GimpMetadataLoadFlags *flags,
+          GimpProcedureConfig   *config,
+          gpointer               run_data)
 {
   GimpValueArray *return_vals;
   GimpImage      *image;
@@ -753,72 +768,48 @@ psp_load (GimpProcedure        *procedure,
 }
 
 static GimpValueArray *
-psp_save (GimpProcedure        *procedure,
-          GimpRunMode           run_mode,
-          GimpImage            *image,
-          gint                  n_drawables,
-          GimpDrawable        **drawables,
-          GFile                *file,
-          const GimpValueArray *args,
-          gpointer              run_data)
+psp_export (GimpProcedure        *procedure,
+            GimpRunMode           run_mode,
+            GimpImage            *image,
+            GFile                *file,
+            GimpExportOptions    *options,
+            GimpMetadata         *metadata,
+            GimpProcedureConfig  *config,
+            gpointer              run_data)
 {
-  GimpProcedureConfig *config;
-  GimpPDBStatusType    status = GIMP_PDB_SUCCESS;
-  GimpExportReturn     export = GIMP_EXPORT_CANCEL;
-  GError              *error  = NULL;
+  GimpPDBStatusType  status = GIMP_PDB_SUCCESS;
+  GimpExportReturn   export = GIMP_EXPORT_IGNORE;
+  GList             *drawables;
+  gint               n_drawables;
+  GError            *error  = NULL;
 
   gegl_init (NULL, NULL);
 
-  config = gimp_procedure_create_config (procedure);
-  gimp_procedure_config_begin_run (config, image, run_mode, args);
-
-  switch (run_mode)
-    {
-    case GIMP_RUN_INTERACTIVE:
-    case GIMP_RUN_WITH_LAST_VALS:
-      gimp_ui_init (PLUG_IN_BINARY);
-
-      export = gimp_export_image (&image, &n_drawables, &drawables, "PSP",
-                                  GIMP_EXPORT_CAN_HANDLE_RGB     |
-                                  GIMP_EXPORT_CAN_HANDLE_GRAY    |
-                                  GIMP_EXPORT_CAN_HANDLE_INDEXED |
-                                  GIMP_EXPORT_CAN_HANDLE_ALPHA   |
-                                  GIMP_EXPORT_CAN_HANDLE_LAYERS);
-
-      if (export == GIMP_EXPORT_CANCEL)
-        return gimp_procedure_new_return_values (procedure,
-                                                 GIMP_PDB_CANCEL,
-                                                 NULL);
-      break;
-
-    default:
-      break;
-    }
-
   if (run_mode == GIMP_RUN_INTERACTIVE)
     {
+      gimp_ui_init (PLUG_IN_BINARY);
+
       if (! save_dialog (procedure, G_OBJECT (config), image))
         status = GIMP_PDB_CANCEL;
     }
 
+  export      = gimp_export_options_get_image (options, &image);
+  drawables   = gimp_image_list_layers (image);
+  n_drawables = g_list_length (drawables);
+
   if (status == GIMP_PDB_SUCCESS)
     {
-      if (! save_image (file, image, n_drawables, drawables,
-                        G_OBJECT (config), &error))
+      if (! export_image (file, image, n_drawables, drawables,
+                          G_OBJECT (config), &error))
         {
           status = GIMP_PDB_EXECUTION_ERROR;
         }
     }
 
-  gimp_procedure_config_end_run (config, status);
-  g_object_unref (config);
-
   if (export == GIMP_EXPORT_EXPORT)
-    {
-      gimp_image_delete (image);
-      g_free (drawables);
-    }
+    gimp_image_delete (image);
 
+  g_list_free (drawables);
   return gimp_procedure_new_return_values (procedure, status, error);
 }
 
@@ -827,22 +818,17 @@ save_dialog (GimpProcedure *procedure,
              GObject       *config,
              GimpImage     *image)
 {
-  GtkWidget    *dialog;
-  GtkListStore *store;
-  GtkWidget    *frame;
-  gint          run;
+  GtkWidget *dialog;
+  GtkWidget *frame;
+  gint       run;
 
-  dialog = gimp_save_procedure_dialog_new (GIMP_SAVE_PROCEDURE (procedure),
-                                           GIMP_PROCEDURE_CONFIG (config),
-                                           image);
+  dialog = gimp_export_procedure_dialog_new (GIMP_EXPORT_PROCEDURE (procedure),
+                                             GIMP_PROCEDURE_CONFIG (config),
+                                             image);
 
   /*  file save type  */
-  store = gimp_int_store_new (_("None"), PSP_COMP_NONE,
-                              _("RLE"),  PSP_COMP_RLE,
-                              _("LZ77"), PSP_COMP_LZ77,
-                              NULL);
-  frame = gimp_procedure_dialog_get_int_radio (GIMP_PROCEDURE_DIALOG (dialog),
-                                               "compression", GIMP_INT_STORE (store));
+  frame = gimp_procedure_dialog_get_widget (GIMP_PROCEDURE_DIALOG (dialog),
+                                            "compression", GIMP_TYPE_INT_RADIO_FRAME);
   gtk_container_set_border_width (GTK_CONTAINER (frame), 12);
 
   gimp_procedure_dialog_fill (GIMP_PROCEDURE_DIALOG (dialog), NULL);
@@ -1104,6 +1090,7 @@ read_creator_block (FILE      *f,
   guint16       keyword;
   guint32       length;
   gchar        *string;
+  gchar        *string2;
   gchar        *title = NULL, *artist = NULL, *copyright = NULL, *description = NULL;
   guint32       dword;
   guint32       __attribute__((unused))cdate = 0;
@@ -1134,7 +1121,17 @@ read_creator_block (FILE      *f,
         }
       keyword = GUINT16_FROM_LE (keyword);
       length = GUINT32_FROM_LE (length);
-      switch (keyword)
+
+      if ((goffset) ftell (f) + length > (goffset) data_start + total_len)
+        {
+          /* FIXME: After string freeze is over, we should consider changing
+           * this error message to be a bit more descriptive. */
+          g_set_error (error, G_FILE_ERROR, G_FILE_ERROR_FAILED,
+                        _("Error reading creator keyword data"));
+          return -1;
+        }
+
+        switch (keyword)
         {
         case PSP_CRTR_FLD_TITLE:
         case PSP_CRTR_FLD_ARTIST:
@@ -1150,6 +1147,19 @@ read_creator_block (FILE      *f,
             }
           /* PSP does not zero terminate strings */
           string[length] = '\0';
+          string2 = string;
+          /* Strings are in ASCII format according to PSP8 specs. */
+          string = g_convert (string2, -1, "utf-8", "iso8859-1", NULL, NULL, NULL);
+          if (string)
+            g_free (string2);
+          else
+            string = string2;
+          if (! g_utf8_validate (string, -1, NULL))
+            {
+              g_printerr ("Invalid creator keyword ignored.\n");
+              break;
+            }
+
           switch (keyword)
             {
             case PSP_CRTR_FLD_TITLE:
@@ -1279,8 +1289,17 @@ read_color_block (FILE      *f,
     }
 
   color_palette_entries = GUINT32_FROM_LE (entry_count);
+  /* TODO: GIMP currently only supports a maximum of 256 colors
+   * in an indexed image. If this changes, we can change this check */
+  if (color_palette_entries > 256)
+    {
+      g_set_error (error, G_FILE_ERROR, G_FILE_ERROR_FAILED,
+                   _("Error: Unsupported palette size"));
+      return -1;
+    }
+
   /* psp color palette entries are stored as RGBA so 4 bytes per entry
-     where the fourth bytes is always zero */
+   * where the fourth bytes is always zero */
   pal_size = color_palette_entries * 4;
   color_palette = g_malloc (pal_size);
   if (fread (color_palette, pal_size, 1, f) < 1)
@@ -1306,7 +1325,7 @@ read_color_block (FILE      *f,
       memcpy (color_palette, tmpmap, color_palette_entries * 3);
       g_free (tmpmap);
 
-      gimp_image_set_colormap (image, color_palette, color_palette_entries);
+      gimp_palette_set_colormap (gimp_image_get_palette (image), babl_format ("R'G'B' u8"), color_palette, color_palette_entries * 3);
       g_free (color_palette);
     }
 
@@ -1427,7 +1446,7 @@ blend_mode_name (PSPBlendModes mode)
   };
   static gchar *err_name = NULL;
 
-  if (mode >= 0 && mode <= PSP_BLEND_TRUE_LIGHTNESS)
+  if (mode <= PSP_BLEND_TRUE_LIGHTNESS)
     return blend_mode_names[mode];
   else if (mode == PSP_BLEND_ADJUST)
     return blend_mode_names[PSP_BLEND_TRUE_LIGHTNESS+1];
@@ -1454,7 +1473,7 @@ layer_type_name (PSPLayerTypePSP6 type)
   };
   static gchar *err_name = NULL;
 
-  if (type >= 0 && type <= keGLTArtMedia)
+  if (type <= keGLTArtMedia)
     return layer_type_names[type];
 
   g_free (err_name);
@@ -1604,9 +1623,14 @@ read_channel_data (FILE        *f,
                 {
                   guchar *p, *q;
 
-                  fread (buf, width, 1, f);
+                  if (fread (buf, 1, width, f) != width)
+                    {
+                      g_set_error (error, G_FILE_ERROR, G_FILE_ERROR_FAILED,
+                                   _("Error reading data. Most likely unexpected end of file."));
+                      return -1;
+                    }
                   /* Contrary to what the PSP specification seems to suggest
-                    scanlines are not stored on a 4-byte boundary. */
+                     scanlines are not stored on a 4-byte boundary. */
                   p = buf;
                   q = pixels[y] + offset;
                   for (i = 0; i < width; i++)
@@ -1622,9 +1646,14 @@ read_channel_data (FILE        *f,
                 {
                   guint16 *p, *q;
 
-                  fread (buf, width * ia->bytes_per_sample, 1, f);
+                  if (fread (buf, ia->bytes_per_sample, width, f) != width)
+                    {
+                      g_set_error (error, G_FILE_ERROR, G_FILE_ERROR_FAILED,
+                                   _("Error reading data. Most likely unexpected end of file."));
+                      return -1;
+                    }
                   /* Contrary to what the PSP specification seems to suggest
-                    scanlines are not stored on a 4-byte boundary. */
+                     scanlines are not stored on a 4-byte boundary. */
                   p = (guint16 *) buf;
                   q = (guint16 *) (pixels[y] + offset);
                   for (i = 0; i < width; i++)
@@ -1649,7 +1678,7 @@ read_channel_data (FILE        *f,
         else
           endq = q + line_width * height;
 
-        buf = g_malloc (127);
+        buf = g_malloc (128);
         while (q < endq)
           {
             fread (&runcount, 1, 1, f);
@@ -2162,11 +2191,12 @@ read_layer_block (FILE      *f,
                 }
               else
                 {
-                  if (channel_type > PSP_CHANNEL_BLUE)
+                  if ((ia->base_type == GIMP_RGB && channel_type > PSP_CHANNEL_BLUE) ||
+                      (ia->base_type != GIMP_RGB && channel_type >= PSP_CHANNEL_RED))
                     {
                       g_set_error (error, G_FILE_ERROR, G_FILE_ERROR_FAILED,
-                                  _("Invalid channel type %d in channel information chunk"),
-                                  channel_type);
+                                   _("Invalid channel type %d in channel information chunk"),
+                                   channel_type);
                       return NULL;
                     }
 
@@ -2353,6 +2383,248 @@ compression_name (gint compression)
   return NULL;
 }
 
+static gint
+read_selection_block (FILE      *f,
+                      GimpImage *image,
+                      guint      total_len,
+                      PSPimage  *ia,
+                      GError   **error)
+{
+  gsize   current_location;
+  gsize   file_size;
+  guint32 chunk_size;
+  guint32 rect[4];
+
+  current_location = ftell (f);
+  fseek (f, 0, SEEK_END);
+  file_size = ftell (f);
+  fseek (f, current_location, SEEK_SET);
+
+  if (fread (&chunk_size, 4, 1, f) < 1)
+    {
+      g_set_error (error, G_FILE_ERROR, G_FILE_ERROR_FAILED,
+                   _("Error reading selection chunk"));
+      return -1;
+    }
+  chunk_size = GUINT32_FROM_LE (chunk_size);
+
+  current_location = ftell (f);
+  if (chunk_size > (file_size - current_location))
+    {
+      g_set_error (error, G_FILE_ERROR, G_FILE_ERROR_FAILED,
+                   _("Invalid selection chunk size"));
+      return -1;
+    }
+
+  if (fread (&rect, 16, 1, f) < 1)
+    {
+      g_set_error (error, G_FILE_ERROR, G_FILE_ERROR_FAILED,
+                   _("Error reading selection chunk"));
+      return -1;
+    }
+
+  swab_rect (rect);
+  gimp_image_select_rectangle (image, GIMP_CHANNEL_OP_ADD,
+                               rect[0], rect[1],
+                               rect[2] - rect[0],
+                               rect[3] - rect[1]);
+
+  /* The file format specifies multiple selections, but
+   * as of PSP 8 they only allow one. Skipping the remaining
+   * information in the block. */
+  total_len -= sizeof (guint32) + sizeof (rect);
+  if (try_fseek (f, total_len, SEEK_SET, error) < 0)
+    {
+      g_set_error (error, G_FILE_ERROR, G_FILE_ERROR_FAILED,
+                   _("Error reading end of selection chunk"));
+      return -1;
+    }
+
+  return 0;
+}
+
+static gint
+read_extended_block (FILE      *f,
+                     GimpImage *image,
+                     guint      total_len,
+                     PSPimage  *ia,
+                     GError   **error)
+{
+  guchar   header[4];
+  guint16  field_type;
+  guint32  field_length;
+
+  while (total_len > 0)
+    {
+      if (fread (header, 4, 1, f) < 1      ||
+          fread (&field_type, 2, 1, f) < 1 ||
+          fread (&field_length, 4, 1, f) < 1)
+        {
+          g_set_error (error, G_FILE_ERROR, G_FILE_ERROR_FAILED,
+                       _("Error reading extended block chunk header"));
+          return -1;
+        }
+
+      if (memcmp (header, "~FL\0", 4) != 0)
+        {
+          g_print ("Header: %s\n", header);
+          g_set_error (error, G_FILE_ERROR, G_FILE_ERROR_FAILED,
+                       _("Invalid extended block chunk header"));
+          return -1;
+        }
+      total_len -= sizeof (header) + sizeof (field_type) + sizeof (field_length);
+
+      field_type   = GUINT16_FROM_LE (field_type);
+      field_length = GUINT32_FROM_LE (field_length);
+
+      if (field_length > total_len)
+        {
+          g_set_error (error, G_FILE_ERROR, G_FILE_ERROR_FAILED,
+                       _("Invalid extended block chunk size"));
+          return -1;
+        }
+
+      switch (field_type)
+        {
+        case PSP_XDATA_GRID:
+          {
+            GeglColor *color;
+            guchar     rgb[4];
+            guint32    h_spacing;
+            guint32    v_spacing;
+            guint16    unit; /* Unused */
+
+            if (field_length != 14              ||
+                fread (rgb, 4, 1, f) < 1        ||
+                fread (&h_spacing, 4, 1, f) < 1 ||
+                fread (&v_spacing, 4, 1, f) < 1 ||
+                fread (&unit, 2, 1, f) < 1)
+              {
+                g_set_error (error, G_FILE_ERROR, G_FILE_ERROR_FAILED,
+                           _("Error reading extended chunk grid data"));
+                return -1;
+              }
+
+            h_spacing = GUINT32_FROM_LE (h_spacing);
+            v_spacing = GUINT32_FROM_LE (v_spacing);
+            gimp_image_grid_set_spacing (image, h_spacing, v_spacing);
+
+            color = gegl_color_new (NULL);
+            gegl_color_set_pixel (color, babl_format ("R'G'B' u8"), rgb);
+            gimp_image_grid_set_foreground_color (image, color);
+            gimp_image_grid_set_background_color (image, color);
+            g_object_unref (color);
+          }
+          break;
+
+        case PSP_XDATA_GUIDE:
+          {
+            guchar  rgb[4]; /* Unused */
+            guint32 offset;
+            guint16 orientation;
+
+            if (field_length != 10           ||
+                fread (rgb, 4, 1, f) < 1     ||
+                fread (&offset, 4, 1, f) < 1 ||
+                fread (&orientation, 2, 1, f) < 1)
+              {
+                g_set_error (error, G_FILE_ERROR, G_FILE_ERROR_FAILED,
+                           _("Error reading extended chunk guide data"));
+                return -1;
+              }
+
+            offset      = GUINT32_FROM_LE (offset);
+            orientation = GUINT16_FROM_LE (orientation);
+
+            if (orientation == keHorizontalGuide)
+              {
+                gimp_image_add_hguide (image, offset);
+              }
+            else if (orientation == keVerticalGuide)
+              {
+                gimp_image_add_vguide (image, offset);
+              }
+            else
+              {
+                g_set_error (error, G_FILE_ERROR, G_FILE_ERROR_FAILED,
+                             _("Invalid guide orientation"));
+                return -1;
+              }
+          }
+          break;
+
+        /* Not yet implemented */
+        case PSP_XDATA_TRNS_INDEX:
+        case PSP_XDATA_EXIF:
+        case PSP_XDATA_IPTC:
+        default:
+          if (try_fseek (f, field_length, SEEK_CUR, error) < 0)
+            {
+              g_set_error (error, G_FILE_ERROR, G_FILE_ERROR_FAILED,
+                           _("Error reading extended block chunk"));
+              return -1;
+            }
+          break;
+        }
+      total_len -= field_length;
+    }
+
+  return 0;
+}
+
+static gint
+read_colorprofile_block (FILE      *f,
+                         GimpImage *image,
+                         guint      total_len,
+                         PSPimage  *ia,
+                         GError   **error)
+{
+  guint             psp_header_size;
+  guint             icc_profile_size;
+  gchar            *icc_profile;
+  GimpColorProfile *profile;
+
+  if (fread (&psp_header_size, 4, 1, f) < 1)
+    {
+      g_set_error (error, G_FILE_ERROR, G_FILE_ERROR_FAILED,
+                   _("Error reading colorprofile chunk"));
+      return -1;
+    }
+  psp_header_size = GUINT32_FROM_LE (psp_header_size);
+
+  /* The next part is a 2 byte length value, followed by PSP's
+   * internal name for the profile. We'll skip most of it,
+   * except for the last 4 bytes which is the profile size */
+  if (try_fseek (f, psp_header_size - 8, SEEK_CUR, error) < 0 ||
+      fread (&icc_profile_size, 4, 1, f) < 1)
+    {
+      g_set_error (error, G_FILE_ERROR, G_FILE_ERROR_FAILED,
+                   _("Error reading colorprofile chunk"));
+      return -1;
+    }
+  icc_profile_size = GUINT32_FROM_LE (icc_profile_size);
+
+  /* The rest of the code references load_resource_1039 ()
+   * in psd-image-res-load.c */
+  icc_profile = g_malloc (icc_profile_size);
+  if (fread (icc_profile, icc_profile_size, 1, f) < 1)
+    {
+      g_set_error (error, G_FILE_ERROR, G_FILE_ERROR_FAILED,
+                   _("Error reading colorprofile chunk"));
+      return -1;
+    }
+
+  profile = gimp_color_profile_new_from_icc_profile ((guint8 *) icc_profile,
+                                                     icc_profile_size,
+                                                     NULL);
+  gimp_image_set_color_profile (image, profile);
+
+  g_object_unref (profile);
+  g_free (icc_profile);
+
+  return 0;
+}
+
 /* The main function for loading PSP-images
  */
 static GimpImage *
@@ -2491,7 +2763,10 @@ load_image (GFile   *file,
               break;
 
             case PSP_SELECTION_BLOCK:
-              break;            /* Not yet implemented */
+              if (read_selection_block (f, image, block_total_len, &ia,
+                                        error) == -1)
+                goto error;
+              break;
 
             case PSP_ALPHA_BANK_BLOCK:
               break;            /* Not yet implemented */
@@ -2500,7 +2775,10 @@ load_image (GFile   *file,
               break;            /* No use for it */
 
             case PSP_EXTENDED_DATA_BLOCK:
-              break;            /* Not yet implemented */
+              if (read_extended_block (f, image, block_total_len, &ia,
+                                       error) == -1)
+                goto error;
+              break;
 
             case PSP_TUBE_BLOCK:
               if (read_tube_block (f, image, block_total_len, &ia, error) == -1)
@@ -2523,7 +2801,10 @@ load_image (GFile   *file,
               break;            /* Not yet implemented */
 
             case PSP_COLORPROFILE_BLOCK:
-              break;            /* Not yet implemented */
+              if (read_colorprofile_block (f, image, block_total_len, &ia,
+                                           error) == -1)
+                goto error;
+              break;
 
             case PSP_LAYER_BLOCK:
             case PSP_CHANNEL_BLOCK:
@@ -2577,12 +2858,12 @@ load_image (GFile   *file,
 }
 
 static gint
-save_image (GFile         *file,
-            GimpImage     *image,
-            gint           n_drawables,
-            GimpDrawable **drawables,
-            GObject       *config,
-            GError       **error)
+export_image (GFile         *file,
+              GimpImage     *image,
+              gint           n_drawables,
+              GList         *drawables,
+              GObject       *config,
+              GError       **error)
 {
   g_set_error (error, G_FILE_ERROR, G_FILE_ERROR_FAILED,
                _("Exporting not implemented yet."));
