@@ -101,7 +101,7 @@ struct _BrowserClass
  */
 
 #define BROWSER_TYPE  (browser_get_type ())
-#define BROWSER (obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BROWSER_TYPE, Browser))
+#define BROWSER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BROWSER_TYPE, Browser))
 
 GType                   browser_get_type         (void) G_GNUC_CONST;
 
@@ -110,7 +110,7 @@ static GimpProcedure  * browser_create_procedure (GimpPlugIn           *plug_in,
                                                   const gchar          *name);
 
 static GimpValueArray * browser_run              (GimpProcedure        *procedure,
-                                                  const GimpValueArray *args,
+                                                  GimpProcedureConfig  *config,
                                                   gpointer              run_data);
 
 static GtkWidget * browser_dialog_new             (void);
@@ -168,7 +168,7 @@ browser_create_procedure (GimpPlugIn  *plug_in,
 
       gimp_procedure_set_menu_label (procedure, _("_Plug-In Browser"));
       gimp_procedure_set_icon_name (procedure, GIMP_ICON_PLUGIN);
-      gimp_procedure_add_menu_path (procedure, "<Image>/Help");
+      gimp_procedure_add_menu_path (procedure, "<Image>/Help/[Programming]");
 
       gimp_procedure_set_documentation (procedure,
                                         _("Display information about plug-ins"),
@@ -186,12 +186,12 @@ browser_create_procedure (GimpPlugIn  *plug_in,
                                       "Andy Thomas",
                                       "1999");
 
-      GIMP_PROC_ARG_ENUM (procedure, "run-mode",
-                          "Run mode",
-                          "The run mode",
-                          GIMP_TYPE_RUN_MODE,
-                          GIMP_RUN_INTERACTIVE,
-                          G_PARAM_READWRITE);
+      gimp_procedure_add_enum_argument (procedure, "run-mode",
+                                        "Run mode",
+                                        "The run mode",
+                                        GIMP_TYPE_RUN_MODE,
+                                        GIMP_RUN_INTERACTIVE,
+                                        G_PARAM_READWRITE);
     }
 
   return procedure;
@@ -199,7 +199,7 @@ browser_create_procedure (GimpPlugIn  *plug_in,
 
 static GimpValueArray *
 browser_run (GimpProcedure        *procedure,
-             const GimpValueArray *args,
+             GimpProcedureConfig  *config,
              gpointer              run_data)
 {
   browser_dialog_new ();
@@ -361,6 +361,7 @@ browser_search (GimpBrowser   *gimp_browser,
                 gint           search_type,
                 PluginBrowser *browser)
 {
+  GimpProcedure  *procedure;
   GimpValueArray *return_vals;
   const gchar   **procedure_strs;
   gint            num_plugins = 0;
@@ -371,10 +372,11 @@ browser_search (GimpBrowser   *gimp_browser,
   gimp_browser_show_message (GIMP_BROWSER (browser->browser),
                              _("Searching by name"));
 
-  return_vals = gimp_pdb_run_procedure (gimp_get_pdb (),
-                                        "gimp-plug-ins-query",
-                                        G_TYPE_STRING, search_text,
-                                        G_TYPE_NONE);
+  procedure   = gimp_pdb_lookup_procedure (gimp_get_pdb (),
+                                           "gimp-plug-ins-query");
+  return_vals = gimp_procedure_run (procedure,
+                                    "search-string", search_text,
+                                    NULL);
 
   if (GIMP_VALUES_GET_ENUM (return_vals, 0) == GIMP_PDB_SUCCESS)
     {
@@ -421,9 +423,9 @@ browser_search (GimpBrowser   *gimp_browser,
       const gint        *time_ints;
       gint               i;
 
-      accel_strs     = GIMP_VALUES_GET_STRV (return_vals, 2);
-      prog_strs      = GIMP_VALUES_GET_STRV (return_vals, 3);
-      time_ints      = GIMP_VALUES_GET_INT32_ARRAY  (return_vals, 5);
+      accel_strs = GIMP_VALUES_GET_STRV (return_vals, 2);
+      prog_strs  = GIMP_VALUES_GET_STRV (return_vals, 3);
+      time_ints  = GIMP_VALUES_GET_INT32_ARRAY (return_vals, 4, NULL);
 
       for (i = 0; i < num_plugins; i++)
         {
